/*
 ***** BEGIN LICENSE BLOCK *****
 * Version: EPL 1.0/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Eclipse Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.eclipse.org/legal/epl-v10.html
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Copyright (C) 2013-2017 The JRuby Team (jruby@jruby.org)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the EPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the EPL, the GPL or the LGPL.
 ***** END LICENSE BLOCK *****/
package org.jruby.ext.ripper;

public class YyTables {
   private static short[] combine(short[] t1, short[] t2, 
                                  short[] t3, short[] t4) {
      short[] t = new short[t1.length + t2.length + t3.length + t4.length];
      int index = 0;
      System.arraycopy(t1, 0, t, index, t1.length);
      index += t1.length;
      System.arraycopy(t2, 0, t, index, t2.length);
      index += t2.length;
      System.arraycopy(t3, 0, t, index, t3.length);
      index += t3.length;
      System.arraycopy(t4, 0, t, index, t4.length);
      return t;
   }

   public static final short[] yyTable() {
      return combine(yyTable1(), yyTable2(), yyTable3(), yyTable4());
   }

   public static final short[] yyCheck() {
      return combine(yyCheck1(), yyCheck2(), yyCheck3(), yyCheck4());
   }
   private static final short[] yyTable1() {
      return new short[] {

           88,   88,  134,  134,  308,  217,  311,  253,  253,  253, 
          133,  133,  279,  253,  253,  263,  381,  253,  355,  421, 
          133,  423,  231,  218,  218,  716,  233,  606,    8,  310, 
          631,  673,  430,  304,  305,  291,  291,  697,    8,  291, 
          785,  645,  231,   88,  218,  468,  233,  353,  107,  253, 
          443,  363,   83,   83,  646,  133,  295,  295,  619,  715, 
          295,  602,  715,  610,  716,  632,  615,  648,  703,  218, 
          692,  307,  644,  259,  644,  490,  259,  571,  251,  251, 
          251,  529,  565,  632,  706,  531,  827,    8,  334,  338, 
          647,  740,  571,  644,  565,  390,  438,  353,  725,  756, 
          836,  580,  275,  916,  602,  133,  610,  644,  263,  644, 
           83,  974,  747,  327,  871,  945,  580,  811,  435,  436, 
          647,  259,  258,  644,  304,  258,  253,  977,   88,   86, 
           74,  848,   73,  758,  294,  294,  673,  517,  294,  603, 
         1029,  379,  644,  737,  259,  317,   72,  433,  435,  820, 
           17,  859,  357,  434,  647,  832,  644,  644,  644,  830, 
          346,   83,  565,  300,   86,  644,  333,  644,  639,  307, 
          258,  379,  379,  303,  415,  644,  307,  644,  571,  647, 
          778,   72, 1035,   71,  464,  883,  639,  106,  106,  257, 
          260,  890,  313,  258,  106,  106,  106,  440,  579,   17, 
          106,  106,  743,  526,  106,  526,  644,  590,  452,  346, 
          478,  565,  571,  579,  796,  313,  453,  941,  751,  565, 
          277,  860,  590,  380,  644,  550,  951,  571,  503,  439, 
          106,   53,  303,  383,  318,   88,  106,  525,  673,  528, 
         1029,  425,  598, 1064,  626,  276,  512,  644,  749,  387, 
          388,  376,  752,  380,  380,  977,  716,  368,  396,  784, 
          381,   75,  253,  253,  263,  370,  384,  792,  379,  382, 
           81,  377, 1085,  415,  550,  945,  598,  380,  490,  441, 
          916,  374,  417,  442,  537,  533,  603,  376,  802,  418, 
          422,  598,  715,  715,  378,  427,  504,  505,  253,  431, 
          253,  253,  855,  469,  253,  955,  253,  822,  819,   79, 
          959,   88,   75,  106,  426,  106,  898,   88,  844,   88, 
          291,   81,  291,  291,  554,  588,  673,  263,  253,  555, 
          459,  644,  896,  251,  251,  603,  815,  561,  306,   76, 
          565,  295,  562,  295,   95,   95,  432,  700,  479,  481, 
          353,  387,  388,  479,  481,  455,  281,  987,  133,  588, 
           79,  661,  662,  644,   72,  565,  519,  645,  644,  515, 
          644,  578,  520,  645,  588,  500,  645,  502,  459,   83, 
          253,  253,  253,  253,   88,  253,  253,   95,  458,  327, 
           76,    8,  571,  631,  819,  390,  470, 1008,  628,  523, 
          792,  905,   17,   17,   17,  578,  619,  644,   17,   17, 
          565,   17,   92,   92,  135,  135,  580,  444,  253,  294, 
          578,  294,  106,  253,  280,  253,   88,  353,  632,  253, 
          253,  937,   88,  612,  275,  133,  569,  275,  611,  603, 
          291,  611,   83,  473,  474,  253,  644,   88,  253,  106, 
          106,  251,  251,  251,  251,   92,  570,  571,   88,  354, 
          612,  253,   74, 1058,   73,  611,   88,  291, 1087,  102, 
          102,  533,   95,  571,  716,  253,  612,  702,  702,  636, 
          846,  611,   17,  291,   83,  106,  833,  106,  106,  644, 
          612,  106,  644,  106,  840,  611,  327,  291,  106,  644, 
          306,  673, 1010,   17,  106,  253,  106,   88,  723,  354, 
          571,  715,  102,  579,  792,  106,  792,  972,   88,  649, 
          550,  231,  590,  218,  612,  233,  616,  714,  620,  611, 
           75,   84,  630,  634,  253,   17,  353,  598,  420,   81, 
           92,  539,  540,  571,  133,   77,  625,  313,  542,  612, 
          625,  565,  277,   82,  611,  277,  291,  550,  396,   83, 
          533,  565,  566,  567,  568,  383,  733,  106,  106,  106, 
          106,  106,  106,  106,  598,  757,  724,  276,   79,   95, 
          276,  603,   84,  376,  764,  792, 1037,  539,  540,  698, 
          550,  598,  381,   75,  542,  465,   77,  102,  384,   80, 
          379,  382,   81,  377,   82,  106, 1042,  598,   76,  380, 
          106,  322,  106,  106,  761,  762,  106,  106,  603,  106, 
          588,  840,  767,  768,  764,  353,  378,  767,  771,   78, 
          644,  466,  106,  133,  106,  106,  644,  644,  253,   88, 
          467,   79,  644,  782,  792,  106,  792,   92,  106,  470, 
           80,  603, 1074,  106,  472,   95,  761,  588,  475,  767, 
          479,   95,  106,   95,  110,  485,  578,  847,   88,  863, 
          764,   76,  486,  801,  588,  792,  489,  865,  447,  644, 
           78,  813,  644,  644,  764,  814,  509,  389,  647,  513, 
          588,  514,  106,  518,  106,  524,  322,   96,  231,  218, 
          218,  448,  233,  578,  102,  106,  530,  253,  532,  251, 
          644, 1005,  533,  110,  891,  764,  538, 1007,   97,  549, 
          912,  106,  915,   92,  909,  804,  806,   88,   95,   92, 
          917,   92,  353,  655,  629,  288,  578,  553,  656,  557, 
          133,  560,  332,  336,  501,  514,  644,  637,  507,  644, 
          448,  684,   88,  412,   88,  687,   88,  253,  253,  644, 
          841,  689,  354,  253,  253,  520,  521,  522,  699,  624, 
           95,  661,  662,  253,  904,  417,   95,  353,  251,   83, 
          102,  718,  418,  419,  719,  133,  102,  900,  102,  720, 
          412,   95,  389,  726,  729,  644,   92,  734,  644,  745, 
           84,   16,   95,  625,  292,  298,  750,  412,  299,  759, 
           95,  760,  463,  775,   77,  663,  765,   88,  766,  783, 
          253,  914,   82,  412,  404,  106,  106,  874,  406,  407, 
          776,   88,   88,   88,  850,  851,  779,  852,   92,  354, 
          786,  702,  788,  797,   92,  539,  540,  541,  798,  799, 
           16,   95,  542,  102, 1036,  106,  803,   88,   88,   92, 
          879,  881,   95,   84,  805,  807,  886,  888,   80, 1023, 
           92,  809,  963, 1027, 1020,  810,  620,   77,   92,  510, 
          133, 1028,  253,  819,  644,   82,  825,  322,  837,  644, 
          826,  251,  828,  126,  106,  102,   88,   88,   78, 1039, 
          829,  102,   59,   60,  477, 1038,  831, 1040,  843, 1041, 
          870,  539,  540,  544,  106,  417,  102,  856,  542,   92, 
         1002,  110,  418,  446,   88,  877,  884,  102,  133,  717, 
           92,   80,  901, 1046,  615,  102,  710,  721,  722,  106, 
          906,  106,  126,  106,  106,  106, 1019,  885,  354,  911, 
          106,  106,  922,  116,  133,  730,  926,  938,  448,  931, 
          106,   78,  930,  979,  644,  919,  981,  983,  919,  985, 
          914,  986,  932, 1082,  316,  939,  102,  948,  949, 1081, 
          952, 1083,  514,   95, 1084,  968,   88,  102,  954,  514, 
          514,  956, 1086,  122,  322,  615,  914,  958,  496,  644, 
          644,  929,  520,   88,  106,   88,  644,  106,  964,  520, 
          520,   88,   95,  965,  967, 1095,  615,  975,  106,  106, 
          106,  978,  980, 1011,  874,  982,  688,  874,  116,  874, 
          448,  874,  984, 1012,  696, 1016,  789,  354,  651,  652, 
          653,  654,  122,  253,  106,  106, 1015, 1017,   88,  417, 
          463,   92,  764,   16,   16,   16,  418,  462,  612,   16, 
           16, 1022,   16,  611,  777,  291,  631,  631,  631,  106, 
          914,   95,  631,  631,  438,  631, 1018,  116,  644, 1032, 
           92,  644, 1030,  106,  106, 1034, 1033,  787, 1043,  491, 
         1052,  494, 1066, 1068, 1069, 1070,   95, 1054,   95, 1056, 
           95, 1059,   91, 1065,   94, 1067, 1071,  870,  102,  520, 
          870,  106,  919, 1079,  870,  496,  789,  624,  651,  652, 
          653,  654, 1088,  335, 1090,  536,  631,  874,  644,  874, 
          644,  874,  644,   16,  874,  633,   91,  102,   94,   92, 
         1094,  417,  496,  632,  354,  115,  631,  637,  418,  476, 
          126,  685,  854,   91,   16,   94,  633,  117,  496,  447, 
          856,   95,  842,  637,   92,  874,   92,  631,   92,  639, 
          845,  633,  496,  106,  633,   95,   95,   95,  644,  378, 
          539,  540,  544,  514,  644,  517,   16,  542,  644,  354, 
          106,  483,  106,  366,  115,  137,  102,  731,  106,  631, 
          220,   95,   95,  559,   96,  772,  117,  988,  539,  540, 
          870,  316,  870,  866,  870,  542,  913,  870,  316,  316, 
          547,  102,  118,  102,  619,  102,  519,  644, 1078,   92, 
          106,  496,  107,  907,  908,  106,  918, 1025,  521, 1024, 
           95,   95,  892,   92,   92,   92,  282, 1077,  870,  903, 
          122,    0, 1045, 1051,    0,    0,  290,  290,  755,  755, 
          290,    0,  105,  438, 1004,   91,    0,    0,   95,   92, 
           92,  118,    0,    0,   96,    0,  644,    0,    0,  316, 
            0,    0,  314,  316,  135,  116,  102,  417,  290,  290, 
            0,  356,  358,  755,  418,  511,  105,  539,  540,  546, 
          102,  102,  102,  522,  542,  770,    0,    0,   92,   92, 
          770,  438,  514,  105,   61,   62,   63,   64,  438,  438, 
          875,  519,  876,    0,  878,    0,  102,  102,  427,    0, 
           95,    0, 1003,  644,  644,  644,   92,    0,  427,    0, 
          644,  973,  770,  622,    0,  438,  520,   95,    0,   95, 
          539,  540,  550,  520,  520,   95,  120,  542,  135,  480, 
          481,  482,  417,    0,  484,  102,  102,    0,    0,  418, 
          535,   91,  615,   94,  616,  539,  540,  558,    0,  818, 
            0,  821,  542,  821,    0,  925,    0,  427,  485,  485, 
          485,  821,   95,  102,    0,  485,  417,    0,   92,  934, 
          935,  936,  115,  418, 1031,  120,  104,  862,    0,  651, 
          652,  653,  654,  655,  117,   92,    0,   92,  656,    0, 
          514,    0,  517,   92,   96,  946,  947,  514,  514,  517, 
          517,    0,    0,  615,   91,  616,   94,  524,    0,  119, 
          104,  657,  857,    0,    0,    0,    0,  857,    0,  659, 
          660,  661,  662,    0,  615,  102,  616,  104,  113,    0, 
           92,    0,    0,  519,  971,    0,    0,  126,    0,    0, 
          519,  519,  102,    0,  102,  521,  755,    0,    0,  118, 
          102,  644,  521,  521,    0,  644,  644,  618,  119,  417, 
            0,    0, 1006,    0,    0,  663,  418, 1044,    0,    0, 
          110,  615,  623,  467,  467,  467,  633,  113,    0,  115, 
          467,  438,  924,    0,    0,  928,  316,  102,    0,    0, 
          523,  438,    0,  316,  316,  290,  290,  290,  358,  619, 
          290,  105,  622,  789,    0,  651,  652,  653,  654,  517, 
          522,  290,    0,  290,  290,  623,    0,  522,  522,  514, 
          857,    0,  508,    0,    0,  438,  514,  514,  519,  857, 
          711,  713,  615,    0,  335,  519,  519,  790,    0,  438, 
          438, 1047,  122, 1048,  438,  791,    0,    0,    0, 1049, 
          622,    0,    0,  615,    0,    0,    0,  622,  622,    0, 
          427,  427,  427,  618,  105,  427,  427,  427,    0,  427, 
          713,    0,  438,  335,  473,  473,  473,    0,    0,  427, 
            0,  473,    0,  120,  622,    0, 1080,    0,  427,  427, 
            0,  427,  427,  427,  427,  427,    0,    0,    0,  821, 
          572,  573,  574,  575,  576,  577,  578,  579,  580,  581, 
          582,  583,  584,  585,  586,  587,  588,  589,  590,  591, 
          592,  593,  594,  595,  596,  597,  427,    0,  106,    0, 
            0,  290,    0,  427,  427,    0,  617,  121,  621,    0, 
          427,    0,  290,  635,  524,  104,  618,  288,    0,    0, 
            0,  524,  524,    0,    0,    0,  112,  288,  290,    0, 
            0,  427,  106,  773,    0,  873,  119,    0,    0,    0, 
          111,    0,    0,    0,  290,    0,  617,  695,  635,  106, 
            0,  290,    0,    0,    0,  113,  121,    0,  290,    0, 
            0,  288,    0,  427,  618,    0,  290,  290,  616,    0, 
          290,  618,  618,    0,    0,  112,  288,  618,  104,  619, 
          496,    0,  685,  821,    0,    0,    0,    0,  525,  111, 
          920,    0,    0,    0,  727,  728,  812,  523,  618,  816, 
            0,    0,    0,    0,  523,  523,  290,  516,   98,  290, 
            0,  124,    0,  438,  438,  438,  517,  290,  438,  438, 
          438,  515,  438,  517,  517,  838,  389,  839,    0,  616, 
            0,  438,  438,  438,  713,  335,    0,    0,    0,    0, 
            0,  438,  438,    0,  438,  438,  438,  438,  438,  789, 
          616,  651,  652,  653,  654,  521,  438,  246,  404,    0, 
          124,    0,  406,  407,  408,  409,    0,  246,  438,  438, 
          438,  438,  438,  438,  438,  438,  438,  438,  438,  438, 
          438,  438,    0,  790,  438,  438,  438,  438,    0,    0, 
          438,  895,  622,  438,    0,    0,  438,    0,  438,  290, 
          438,  246,  438,  902,  438,  438,  438,  438,  438,  438, 
          438,    0,  438,  438,  438,  246,  246,    0,    0,    0, 
          246,    0,   99,    0,  101,  942,  438,  651,  652,  653, 
          654,    0,  873,    0,    0,  873,  107, 1026,    0,  873, 
          205,  382,  383,  384,  385,  386,  438,  933,  789,    0, 
          651,  652,  653,  654,    0,    0,    0,    0,    0,  514, 
            0,    0,  290,    0,  121,  290,    0,  106,  619,  522, 
          107,  524,    0,    0,  514,    0,    0,    0,    0,  288, 
          288,  288,  790,  112,  288,  288,  288,  107,  288,  205, 
          976,  290,    0,  290,    0,    0,    0,  111,  288,  288, 
          290,  290,    0,    0,    0,    0,  514,  288,  288,  970, 
          288,  288,  288,  288,  288,    0,  619,  316,    0,    0, 
            0,    0,    0,  619,  619,  525,  514,    0,    0,  619, 
          106,    0,  525,  525,    0,  873,  514,  873,    0,  873, 
          880,  882,  873,  117,  516,    0,  887,  889,    0,    0, 
          619,  516,  516,  335,    0,  893,  695,  290,  515,  288, 
            0,  897,  288,    0,  288,  515,  515,    0,  124,  290, 
          514,    0,    0,  873,    0,    0,    0,    0,    0,    0, 
          288,  622,    0,    0,  514,  514,    0,  110,    0,  514, 
            0,    0,  521,    0,    0,    0,    0,    0,    0,  521, 
          521,    0,    0,    0,    0,    0,  880,  882,    0,  887, 
          889,    0,  288,  290,    0,    0,    0,  514,    0,  246, 
          246,  246,    0,    0,  246,  246,  246,    0,  246,  622, 
            0,    0,    0,    0,    0,    0,  622,  622,  246,  246, 
            0,    0,  618,    0,    0,    0,    0,  246,  246,    0, 
          246,  246,  246,  246,  246,    0,    0,  118,  966,  120, 
            0,    0,  246,  622,    0,  969,  517,    0,    0,    0, 
            0,    0,    0,    0,    0,  290,  517,    0,    0,    0, 
            0,  246,  246,    0,    0,    0,  246,  246,    0,    0, 
            0,    0,    0,    0,    0,  615,  246,    0,    0,  246, 
            0,    0,  246,    0,  246,  107,  522,    0,  524,    0, 
          517,    0,  969,  522,  522,  524,  524,    0,    0,  290, 
          246,    0,    0,    0,  517,  517,    0,  113,    0,  517, 
            0,    0,  246,    0,    0,    0,  205,    0,  205,  205, 
          205,  205,  205,  514,    0,    0,    0,  205,    0,    0, 
          514,  514,  246,    0,  316,    0,  615,  517,  520,    0, 
            0,  316,  316,    0,    0,    0,  520,  619,  107,    0, 
          205,  205,    0,    0,  514,    0,    0,  615,  205,  205, 
          205,  205,    0,  615,  615,  615,    0,  615,  514,  514, 
          514,  615,  615,  514,  514,  514,  615,  514,  615,  615, 
          615,  615,  615,  615,  615,  307,  615,  514,  514,  615, 
          615,  615,  615,  615,  615,  615,  514,  514,  615,  514, 
          514,  514,  514,  514,  205,  615,  290,    0,  615,  615, 
          615,  514,  615,  615,  615,  615,  615,  615,  615,  615, 
          615,  615,  615,  514,  514,  514,  514,  514,  514,  514, 
          514,  514,  514,  514,  514,  514,  514,    0,  414,  514, 
          514,  514,  514,  615,  526,  514,  615,  615,  514,  615, 
          615,  514,  615,  514,  615,  514,  615,  514,  615,  514, 
          514,  514,  514,  514,  514,  514,  615,  514,  615,  514, 
            0,  615,  615,  615,  615,  615,  615,    0,  517,    0, 
          615,  514,  615,  615,    0,  615,  615,  615,  615,    0, 
            0,    0,  545,  517,  548,    0,  551,  114,  615,  615, 
          615,  514,  551,  616,  616,  616,    0,  616,  517,  517, 
          517,  616,  616,  517,  517,  517,  616,  517,  616,  616, 
          616,  616,  616,  616,  616,  517,  616,  517,  517,  616, 
          616,  616,  616,  616,  616,  616,  517,  517,  616,  517, 
          517,  517,  517,  517,    0,  616,  114,    0,  616,  616, 
          616,  517,  616,  616,  616,  616,  616,  616,  616,  616, 
          616,  616,  616,  517,  517,  517,  517,  517,  517,  517, 
          517,  517,  517,  517,  517,  517,  517,  100,  518,  517, 
          517,  517,  517,  616,  529,  517,  616,  616,  517,  616, 
          616,  517,  616,  517,  616,  517,  616,  517,  616,  517, 
          517,  517,  517,  517,  517,  517,  616,  517,  616,  517, 
          545,  616,  616,  616,  616,  616,  616,    0,    0,  622, 
          616,  517,  616,  616,  523,  616,  616,  616,  616,  622, 
            0,  650,    0,  651,  652,  653,  654,  655,  616,  616, 
          616,  517,  656,  632,  632,  632,    0,    0,    0,  632, 
          632,  389,  632,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  622,    0,  657,  658,  402,  403,    0, 
            0,    0,    0,  659,  660,  661,  662,  622,  622,  744, 
          124,    0,  622,  404,    0,  405,  744,  406,  407,  408, 
          409,    0,  389,  390,  391,  392,  393,  394,  395,  396, 
          397,  398,  399,    0,  400,  401,    0,    0,  402,  403, 
          622,    0,    0,    0,  616,    0,    0,    0,    0,  663, 
            0,    0,    0,  632,  404,    0,  405,    0,  406,  407, 
          408,  409,  410,  411,  412,    0,  413,  650,    0,  651, 
          652,  653,  654,  655,  632,    0,    0,    0,  656,    0, 
            0,    0,    0,    0,  114,    0,    0,    0,  618,    0, 
            0,    0,  517,    0,    0,    0,    0,    0,  618,  517, 
          517,  657,  746,    0,    0,  616,  632,    0,    0,  659, 
          660,  661,  662,    0,  650,    0,  651,  652,  653,  654, 
          655,    0,    0,  517,    0,  656,  616,    0,    0,    0, 
            0,    0,  618,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  119,    0,  389,  518,  618,  618,  657,  123, 
            0,  618,  518,  518,    0,  663,  659,  660,  661,  662, 
          402,  403,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  404,    0,  405,  618, 
          406,  407,  408,  409,    0,    0,  412,    0,  413,    0, 
            0,  523,    0,    0,    0,    0,    0,    0,  523,  523, 
          198,    0,  663,    0,    0,    0,  618,  618,  618,    0, 
          618,  622,  622,  622,  618,  618,  622,  622,  622,  618, 
          622,  618,  618,  618,  618,  618,  618,  618,    0,  622, 
          622,  622,  618,  618,  618,  618,  618,  618,  618,  622, 
          622,  618,  622,  622,  622,  622,  622,    0,  618,  198, 
            0,  618,  618,  618,  622,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  622,  622,  622,  622, 
          622,  622,  622,  622,  622,  622,  622,  622,  622,  622, 
           94,    0,  622,  622,  622,  622,  618,  208,  622,  618, 
          618,  622,  618,  618,  622,  618,  622,  618,  622,  618, 
          622,  618,  622,  622,  622,  622,  622,  622,  622,  618, 
          622,  622,  622,    0,  618,  618,  618,  618,  618,  618, 
          103,    0,    0,  618,  622,  618,  618,  517,  618,  618, 
          618,  618,    0,    0,    0,    0,  208,    0,    0,  128, 
            0,  618,  618,  618,  622,  618,  618,  618,    0,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  438,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,    0,  618,  128,    0, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  102, 
          306,  618,  618,  618,  618,  618,    0,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,    0,  618,  618,  618,  618,  618,  618,    0, 
            0,  619,  618,  618,  618,  618,  525,  618,  618,  618, 
          618,  619,  389,    0,    0,    0,    0,    0,    0,    0, 
          618,  618,  618,  618,    0,    0,    0,    0,  402,  403, 
            0,    0,    0,    0,    0,    0,  198,    0,  198,  198, 
          198,  198,  198,    0,  404,  619,    0,  198,  406,  407, 
          408,  409,    0,    0,    0,  113,  616,    0,  519,  619, 
          619,    0,  125,    0,  619,    0,  519,    0,    0,    0, 
          198,  198,    0,    0,    0,    0,    0,    0,  198,  198, 
          198,  198,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  619,    0,    0,  122,  438,  789,    0,  651, 
          652,  653,  654,  655,  517,    0,    0,    0,  656,    0, 
            0,  517,  517,    0,    0,    0,    0,  616,    0,    0, 
            0,    0,    0,  208,  198,  208,  208,  208,  208,  208, 
            0,  657,    0,    0,  208,    0,  128,    0,  616,    0, 
          307,  661,  662,    0,  438,  521,    0,    0,    0,    0, 
          307,  438,  438,  521,    0,    0,    0,  208,  208,    0, 
            0,    0,    0,    0,    0,  208,  208,  208,  208,    0, 
            0,    0,    0,    0,    0,    0,  389,    0,  438,    0, 
            0,  394,  395,    0,  307,  663,    0,    0,    0,    0, 
            0,    0,  402,  403,  121,    0,    0,  306,  307,  307, 
            0,    0,    0,  307,  306,  306,    0,    0,  404,    0, 
          405,  208,  406,  407,  408,  409,  410,  411,  412,    0, 
          413,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  307,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  525,    0,    0,    0,    0,    0,    0, 
          525,  525,  193,    0,    0,    0,    0,    0,  619,  619, 
          619,    0,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
            0,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,    0, 
          619,  193,    0,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  105,    0,  619,  619,  619,  619,  619,  175, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,    0,  619,  619,  619,  619, 
          619,  619,   93,    0,    0,  619,  619,  619,  619,  622, 
          619,  619,  619,  619,    0,    0,    0,    0,  175,    0, 
            0,  127,    0,  619,  619,  619,  619,  617,  617,  617, 
            0,  617,  307,  307,  307,  617,  617,  307,  307,  307, 
          617,  307,  617,  617,  617,  617,  617,  617,  617,  516, 
          617,  307,  307,  617,  617,  617,  617,  617,  617,  617, 
          307,  307,  617,  307,  307,  307,  307,  307,    0,  617, 
          127,    0,  617,  617,  617,  307,  617,  617,  617,  617, 
          617,  617,  617,  617,  617,  617,  617,  307,  307,  307, 
          307,  307,  307,  307,  307,  307,  307,  307,  307,  307, 
          307,   92,  317,  307,  307,  307,  307,  617,    0,  307, 
          617,  617,  307,  617,  617,  307,  617,  307,  617,  307, 
          617,  307,  617,  307,  307,  307,  307,  307,  307,  307, 
          617,  307,  617,  307,    0,  617,  617,  617,  617,  617, 
          617,    0,    0,  623,  617,  307,  617,  617,  515,  617, 
          617,  617,  617,  623,    0,    0,    0,    0,    0,    0, 
            0,    0,  617,  617,  617,  307,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  193,    0, 
          193,  193,  193,  193,  193,    0,    0,  623,    0,  193, 
            0,    0,    0,    0,    0,    0,    0,  124,  622,    0, 
          522,  623,  623,    0,    0,    0,  623,    0,  522,    0, 
            0,    0,  193,  193,    0,    0,    0,    0,    0,    0, 
          193,  193,  193,  193,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  623,    0,    0,  112,    0,    0, 
            0,    0,    0,    0,    0,    0,  622,    0,    0,    0, 
            0,    0,    0,  622,  622,    0,    0,    0,    0,  618, 
            0,    0,    0,    0,    0,  175,  193,  175,  175,  175, 
          175,  175,    0,    0,    0,    0,  175,    0,  127,    0, 
          622,    0,  624,    0,    0,    0,  516,  524,    0,    0, 
            0,    0,  624,  516,  516,  524,    0,    0,    0,  175, 
          175,    0,    0,    0,    0,    0,    0,  175,  175,  175, 
          175,    0,    0,    0,    0,    0,    0,    0,  389,  390, 
          391,  392,  393,  394,  395,  396,  624,  398,  399,    0, 
            0,    0,    0,    0,  402,  403,  111,    0,    0,  317, 
          624,  624,    0,    0,    0,  624,  317,  317,    0,    0, 
          404,    0,  405,  175,  406,  407,  408,  409,  410,  411, 
          412,    0,  413,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  624,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  515,    0,    0,    0,    0, 
            0,    0,  515,  515,  176,    0,    0,    0,    0,    0, 
          620,  620,  620,    0,  620,  623,  623,  623,  620,  620, 
          623,  623,  623,  620,  623,  620,  620,  620,  620,  620, 
          620,  620,    0,  623,  623,  623,  620,  620,  620,  620, 
          620,  620,  620,  623,  623,  620,  623,  623,  623,  623, 
          623,    0,  620,  176,    0,  620,  620,  620,  623,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          623,  623,  623,  623,  623,  623,  623,  623,  623,  623, 
          623,  623,  623,  623,  104,    0,  623,  623,  623,  623, 
          620,  177,  623,  620,  620,  623,  620,  620,  623,  620, 
          623,  620,  623,  620,  623,  620,  623,  623,  623,  623, 
          623,  623,  623,  620,  623,  623,  623,    0,  620,  620, 
          620,  620,  620,  620,   95,    0,    0,  620,  623,  620, 
          620,  618,  620,  620,  620,  620,    0,    0,    0,    0, 
          177,    0,    0,  106,    0,  620,  620,  620,  623,  621, 
          621,  621,    0,  621,  624,  624,  624,  621,  621,  624, 
          624,  624,  621,  624,  621,  621,  621,  621,  621,  621, 
          621,  518,  624,  624,  624,  621,  621,  621,  621,  621, 
          621,  621,  624,  624,  621,  624,  624,  624,  624,  624, 
          619,  621,    0,    0,  621,  621,  621,  624,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  624, 
          624,  624,  624,  624,  624,  624,  624,  624,  624,  624, 
          624,  624,  624,  109,    0,  624,  624,  624,  624,  621, 
            0,  624,  621,  621,  624,  621,  621,  624,  621,  624, 
          621,  624,  621,  624,  621,  624,  624,  624,  624,  624, 
          624,  624,  621,  624,  624,  624,    0,  621,  621,  621, 
          621,  621,  621,    0,    0,  316,  621,  624,  621,  621, 
          306,  621,  621,  621,  621,  316,    0,    0,    0,    0, 
            0,    0,    0,    0,  621,  621,  621,  624,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          176,    0,  176,  176,  176,  176,  176,    0,    0,  316, 
            0,  176,    0,    0,    0,    0,    0,    0,    0,  123, 
          618,    0,  523,  316,  316,    0,  126,    0,  316,    0, 
          523,    0,    0,    0,  176,  176,    0,    0,    0,    0, 
            0,    0,  176,  176,  176,  176,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  316,    0,    0,  114, 
            0,    0,    0,    0,    0,    0,    0,    0,  618,    0, 
            0,    0,    0,    0,    0,  618,  618,    0,  125,  619, 
            0,  618,    0,    0,    0,    0,    0,  177,  176,  177, 
          177,  177,  177,  177,    0,    0,    0,    0,  177,    0, 
            0,    0,  618,    0,  437,    0,    0,    0,  518,  525, 
            0,    0,    0,    0,  437,  518,  518,  525,    0,    0, 
            0,  177,  177,    0,    0,    0,    0,  619,    0,  177, 
          177,  177,  177,    0,  619,  619,    0,    0,    0,    0, 
          619,    0,    0,    0,    0,    0,    0,    0,  437,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  128,    0, 
            0,  619,    0,  437,    0,    0,    0,  437,    0,    0, 
            0,    0,    0,    0,    0,  177,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  437,    0,    0,  316,    0, 
            0,    0,    0,    0,    0,    0,    0,  306,    0,    0, 
            0,    0,    0,  316,  306,  306,  134,    0,    0,    0, 
            0,    0,  619,  619,  619,    0,  619,  316,  316,  316, 
          619,  619,  316,  316,  316,  619,  316,  619,  619,  619, 
          619,  619,  619,  619,    0,  316,  316,  316,  619,  619, 
          619,  619,  619,  619,  619,  316,  316,  619,  316,  316, 
          316,  316,  316,    0,  619,  134,    0,  619,  619,  619, 
          316,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  316,  316,  316,  316,  316,  316,  316,  316, 
          316,  316,  316,  316,  316,  316,  108,    0,  316,  316, 
          316,  316,  619,  135,  316,  619,  619,  316,  619,  619, 
          316,  619,  316,  619,  316,  619,  316,  619,  316,  316, 
          316,  316,  316,  316,  316,  619,  316,    0,  316,    0, 
          619,  619,  619,  619,  619,  619,    0,    0,    0,  619, 
          316,  619,  619,  317,  619,  619,  619,  619,    0,    0, 
            0,    0,  135,    0,    0,    0,    0,  619,  619,  619, 
          316,  283,  283,  283,    0,  283,  437,  437,  437,  283, 
          283,  437,  437,  437,  283,  437,  283,  283,  283,  283, 
          283,  283,  283,    0,  437,  437,  437,  283,  283,  283, 
          283,  283,  283,  283,  437,  437,  283,  437,  437,  437, 
          437,  437,    0,  283,    0,    0,  283,  283,  283,    0, 
          283,  283,  283,  283,  283,  283,  283,  283,  283,  283, 
          283,  437,  437,  437,  437,  437,  437,  437,  437,  437, 
          437,  437,  437,  437,  437,  414,    0,  437,  437,  437, 
          437,  283,    0,  437,  283,    0,  437,  283,  283,  437, 
          283,  437,  283,  437,  283,  437,  283,  437,  437,  437, 
          437,  437,  437,  437,  283,  437,  437,  437,    0,  283, 
          283,  283,  283,  283,  283,    0,    0,  644,  283,    0, 
          283,  283,  316,  283,  283,  283,  283,  644,    0,  316, 
          316,    0,    0,    0,    0,  619,  283,  283,  283,  437, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  134,  316,  134,  134,  134,  134,  134,    0, 
            0,  644,    0,  134,    0,    0,    0,    0,    0,    0, 
            0,  127,    0,    0,  514,    0,  644,    0,    0,    0, 
          644,    0,  514,    0,    0,    0,  134,  134,    0,    0, 
            0,    0,    0,    0,  134,  134,  134,  134,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  644,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          317,    0,    0,    0,    0,    0,    0,  317,  317,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  135, 
          134,  135,  135,  135,  135,  135,    0,    0,    0,    0, 
          135,    0,    0,    0,    0,    0,  323,    0,    0,    0, 
            0,  517,    0,    0,    0,    0,  323,    0,    0,  517, 
            0,    0,    0,  135,  135,    0,    0,    0,    0,    0, 
            0,  135,  135,  135,  135,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          323,    0,    0,    0,    0,    0,    0,    0,    0,  774, 
            0,    0,    0,    0,    0,  323,    0,    0,    0,  323, 
            0,    0,    0,    0,    0,    0,    0,  135,    0,  389, 
          390,  391,  392,  393,  394,  395,  396,  397,  398,  399, 
            0,  400,  401,    0,    0,  402,  403,  323,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  404,    0,  405,    0,  406,  407,  408,  409,  410, 
          411,  412,    0,  413,  283,  283,  283,    0,  283,  644, 
          644,  644,  283,  283,  644,  644,  644,  283,  644,  283, 
          283,  283,  283,  283,  283,  283,    0,  644,  644,  644, 
          283,  283,  283,  283,  283,  283,  283,  644,  644,  283, 
          644,  644,  644,  644,  644,    0,  283,    0,    0,  283, 
          283,  283,    0,  283,  283,  283,  283,  283,  283,  283, 
          283,  283,  283,  283,  644,  644,  644,  644,  644,  644, 
          644,  644,  644,  644,  644,  644,  644,  644,  262,    0, 
          644,  644,  644,  644,  283,    0,  644,  283,  262,  644, 
          283,  283,  644,  283,  644,  283,  644,  283,  644,  283, 
          644,  644,  644,  644,  644,  644,  644,  283,  644,  644, 
          644,    0,  283,  283,  283,  283,  283,  283,  622,    0, 
            0,  283,  262,  283,  283,    0,  283,  283,  283,  283, 
            0,    0,    0,  622,    0,    0,  262,  262,    0,  283, 
          283,  283,  644,  283,  283,  283,    0,  283,  323,  323, 
          323,  283,  283,  323,  323,  323,  283,  323,  283,  283, 
          283,  283,  283,  283,  283,  622,    0,  323,  323,  283, 
          283,  283,  283,  283,  283,  283,  323,  323,  283,  323, 
          323,  323,  323,  323,    0,  283,    0,    0,  283,  283, 
          283,    0,  283,  283,  283,  283,  283,  283,  283,  283, 
          283,  283,  283,  323,  323,  323,  323,  323,  323,  323, 
          323,  323,  323,  323,  323,  323,  323,    0,    0,  323, 
          323,  323,  323,  283,    0,  323,  283,    0,  323,  283, 
          283,  323,  283,  323,  283,  323,  283,  323,  283,  323, 
          323,  323,  323,  323,  323,  323,  283,  323,  514,  323, 
            0,  283,  283,  283,  283,  283,  283,    0,  514,    0, 
          283,    0,  283,  283,    0,  283,  283,  283,  283,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  283,  283, 
          283,  323,  862,    0,  651,  652,  653,  654,  655,    0, 
            0,    0,   91,  656,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  514,    0,  110, 
            0,  514,    0,    0,    0,    0,  657,    0,    0,    0, 
            0,    0,    0,    0,  659,  660,  661,  662,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  514, 
          262,  262,  262,    0,    0,  262,  262,  262,    0,  262, 
            0,    0,    0,    0,  622,    0,    0,    0,    0,  262, 
          262,    0,    0,    0,    0,    0,    0,    0,  262,  262, 
          663,  262,  262,  262,  262,  262,    0,    0,    0,    0, 
            0,    0,    0,  262,    0,    0,    0,  517,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  517,    0,    0, 
            0,    0,  622,    0,    0,    0,    0,    0,    0,  622, 
          622,    0,    0,    0,    0,  618,    0,  262,    0,    0, 
          262,    0,    0,  262,    0,  262,    0,    0,    0,    0, 
            0,   94,    0,  622,    0,    0,  622,    0,    0,    0, 
            0,  262,    0,  613,  608,    0,  517,  614,  113,    0, 
          517,  187,  188,  262,  189,  190,  191,  192,  193,  194, 
          195,    0,    0,  196,  197,    0,    0,    0,  198,  199, 
          200,  201,    0,  262,    0,    0,    0,    0,  517,    0, 
            0,    0,    0,    0,    0,  203,  204,    0,  205,  206, 
          207,  208,  209,  210,  211,  212,  213,  214,  215,    0, 
            0,  216,    0,    0,    0,  615,  615,  615,    0,  615, 
          514,  514,  514,  615,  615,    0,  514,  514,  615,  514, 
          615,  615,  615,  615,  615,  615,  615,    0,  615,    0, 
            0,  615,  615,  615,  615,  615,  615,  615,  514,  514, 
          615,  514,  514,  514,  514,  514,    0,  615,    0,    0, 
          615,  615,  615,    0,  615,  615,  615,  615,  615,  615, 
          615,  615,  615,  615,  615,  514,  514,  514,  514,  514, 
          514,  514,  514,  514,  514,  514,  514,  514,  514,  414, 
            0,  514,  514,  514,  514,  615,  526,  103,  615,  615, 
          514,  615,  615,    0,  615,    0,  615,  514,  615,  514, 
          615,  514,  514,  514,  514,  514,  514,  514,  615,  514, 
          615,  514,    0,  615,  615,  615,  615,  615,  615,    0, 
            0,  103,  615,    0,  615,  615,    0,  615,  615,  615, 
          615,    0,    0,    0,    0,    0,    0,    0,  103,    0, 
          615,  615,  615,  514,  616,  616,  616,    0,  616,  517, 
          517,  517,  616,  616,    0,  517,  517,  616,  517,  616, 
          616,  616,  616,  616,  616,  616,    0,  616,  438,    0, 
          616,  616,  616,  616,  616,  616,  616,  517,  517,  616, 
          517,  517,  517,  517,  517,    0,  616,    0,    0,  616, 
          616,  616,    0,  616,  616,  616,  616,  616,  616,  616, 
          616,  616,  616,  616,  517,  517,  517,  517,  517,  517, 
          517,  517,  517,  517,  517,  517,  517,  517,  414,    0, 
          517,  517,  517,  517,  616,  529,    0,  616,  616,  517, 
          616,  616,    0,  616,    0,  616,  517,  616,  517,  616, 
          517,  517,  517,  517,  517,  517,  517,  616,  517,  616, 
          517,    0,  616,  616,  616,  616,  616,  616,    0,    0, 
          622,  616,    0,  616,  616,    0,  616,  616,  616,  616, 
          622,    0,    0,    0,    0,    0,    0,    0,    0,  616, 
          616,  616,  517,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  105,    0,    0,    0,    0,    0, 
            0,    0,    0,  780,    0,    0,    0,    0,    0,  622, 
            0,  124,    0,  622,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  389,  390,  391,  392,  393,  394,  395, 
          396,  397,  398,  399,    0,  400,  401,    0,    0,  402, 
          403,  622,    0,    0,    0,    0,  103,  438,    0,    0, 
            0,    0,    0,    0,    0,  404,    0,  405,    0,  406, 
          407,  408,  409,  410,  411,  412,    0,  413,    0,    0, 
            0,    0,    0,    0,  389,  390,  391,  392,  393,  394, 
          395,  396,  397,  398,  399,    0,    0,    0,    0,  618, 
          402,  403,    0,    0,    0,  438,    0,    0,    0,  618, 
            0,    0,  438,  438,    0,    0,  404,    0,  405,  103, 
          406,  407,  408,  409,  410,  411,  412,    0,  413,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  438, 
            0,    0,    0,  104,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  618,    0, 
          123,    0,  618,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  389,  390,  391,  392,  393,  394,  395,  396, 
          397,  398,  399,    0,  400,  401,    0,    0,  402,  403, 
          618,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  404,    0,  405,    0,  406,  407, 
          408,  409,  410,  411,  412,    0,  413,  618,  618,  618, 
            0,  618,  622,  622,  622,  618,  618,    0,  622,  622, 
          618,  622,  618,  618,  618,  618,  618,  618,  618, 
      };
   }

   private static final short[] yyTable2() {
      return new short[] {

            0,  622,    0,    0,  618,  618,  618,  618,  618,  618, 
          618,  622,  622,  618,  622,  622,  622,  622,  622,    0, 
          618,    0,    0,  618,  618,  618,    0,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  622,  622, 
          622,  622,  622,  622,  622,  622,  622,  622,  622,  622, 
          622,  622,    0,    0,  622,  622,  622,  622,  618,    0, 
          123,  618,  618,  622,  618,  618,    0,  618,    0,  618, 
          622,  618,  622,  618,  622,  622,  622,  622,  622,  622, 
          622,  618,  622,  622,  622,    0,  618,  618,  618,  618, 
          618,  618,    0,    0,    0,  618,    0,  618,  618,    0, 
          618,  618,  618,  618,  953,    0,    0,    0,  957,  123, 
            0,    0,    0,  618,  618,  618,  622,  618,  618,  618, 
            0,  618,  618,  618,  618,  618,  618,    0,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,    0, 
          618,  618,    0,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,    0,  618, 
            0,    0,  618,  618,  618,    0,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,    0,    0,  618,  618,  618,  618,  618,    0,    0, 
          618,  618,  618,  618,  618,    0,  618,    0,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,    0,  618,  618,  618,  618,  618, 
          618,    0,    0,  619,  618,    0,  618,  618,    0,  618, 
          618,  618,  618,  619,    0,    0,   97,    0,    0,    0, 
            0,    0,  618,  618,  618,  618,    0,    0,    0, 1053, 
            0,    0, 1055, 1057,    0,    0, 1060, 1061,    0,  389, 
          390,  391,  392,  393,  394,  395,    0,  106,  398,  399, 
           97,    0,    0,    0,    0,  402,  403,    0,    0,    0, 
            0,    0,  619,    0,  125,    0,  619,   97,    0,    0, 
            0,  404,    0,  405,    0,  406,  407,  408,  409,  410, 
          411,  412,    0,  413,    0,    0,    0,  123,    0,    0, 
         1089, 1091, 1092, 1093,  619,    0,    0,  520,    0,    0, 
          618,    0,    0,    0, 1096,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  316,    0,    0,    0,    0,    0,  618,    0, 
            0,    0,  316,    0,    0,  618,  618,    0,    0,    0, 
            0,  618,    0,    0,  599,  600,    0,    0,  601,    0, 
            0,    0,  187,  188,    0,  189,  190,  191,  192,  193, 
          194,  195,  618,    0,  196,  197,  107,    0,    0,  198, 
          199,  200,  201,    0,    0,    0,    0,    0,    0,  303, 
            0,  316,    0,  126,    0,  316,  203,  204,    0,  205, 
          206,  207,  208,  209,  210,  211,  212,  213,  214,  215, 
            0,    0,  216,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  316,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          619,  619,  619,    0,  619,  619,  619,  619,  619,  619, 
            0,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,    0,  619,    0,   97,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,    0,  619,    0,    0,  619,  619,  619,    0,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  520,    0,  619,  619,  619,  619, 
          619,  520,  520,  619,  619,  619,  619,  619,   97,  619, 
            0,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,    0,  619,  619, 
          619,  619,  619,  619,    0,    0,    0,  619,    0,  619, 
          619,  644,  619,  619,  619,  619,    0,    0,    0,    0, 
            0,  644,    0,    0,    0,  619,  619,  619,  619,  619, 
          619,  619,    0,  619,  316,  316,  316,  619,  619,    0, 
          316,  316,  619,  316,  619,  619,  619,  619,  619,  619, 
          619,    0,    0,    0,    0,  619,  619,  619,  619,  619, 
          619,  619,  316,  316,  619,  316,  316,  316,  316,  316, 
          644,  619,    0,    0,  619,  619,  619,    0,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  316, 
          316,  316,  316,  316,  316,  316,  316,  316,  316,  316, 
          316,  316,  316,    0,    0,  316,  316,  316,  316,  619, 
            0,    0,  619,  619,  316,  619,  619,    0,  619,    0, 
          619,  316,  619,  316,  619,  316,  316,  316,  316,  316, 
          316,  316,  619,  316,    0,  316,    0,  619,  619,  619, 
          619,  619,  619,    0,    0,  437,  619,    0,  619,  619, 
            0,  619,  619,  619,  619,  437,    0,    0,    0,    0, 
            0,    0,    0,    0,  619,  619,  619,  316,    0,    0, 
          607,  608,    0,    0,  609,    0,    0,    0,  187,  188, 
            0,  189,  190,  191,  192,  193,  194,  195,    0,  437, 
          196,  197,    0,    0,    0,  198,  199,  200,  201,    0, 
            0,    0,    0,  437,  437,  303,    0,    0,  437,    0, 
            0,    0,  203,  204,    0,  205,  206,  207,  208,  209, 
          210,  211,  212,  213,  214,  215,    0,    0,  216,    0, 
            0,    0,    0,    0,    0,    0,  437,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           52,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           52,    0,    0,    0,    0,    0,    0,    0,  283,  283, 
          283,    0,  283,  644,  644,  644,  283,  283,  644,  644, 
          644,  283,  644,  283,  283,  283,  283,  283,  283,  283, 
            0,  644,  644,  644,  283,  283,  283,  283,  283,  283, 
          283,  644,  644,  283,  644,  644,  644,  644,  644,   52, 
          283,    0,    0,  283,  283,  283,    0,  283,  283,  283, 
          283,  283,  283,  283,  283,  283,  283,  283,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  644, 
            0,    0,    0,    0,    0,    0,  644,  644,  283,    0, 
            0,  283,    0,  644,  283,  283,    0,  283,    0,  283, 
            0,  283,    0,  283,    0,    0,    0,    0,    0,    0, 
            0,  283,    0,  644,  644,    0,  283,  283,  283,  283, 
          283,  283,    0,    0,    0,  283,  520,  283,  283,    0, 
          283,  283,  283,  283,    0,    0,  520,    0,    0,    0, 
            0,    0,    0,  283,  283,  283,  644,  437,  437,  437, 
            0,    0,  437,  437,  437,    0,  437,    0,    0,    0, 
            0,    0,    0,    0,    0,  437,  437,  437,    0,    0, 
          520,    0,    0,    0,    0,  437,  437,    0,  437,  437, 
          437,  437,  437,    0,  520,  520,    0,  116,    0,  520, 
          437,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  437,  437,  437,  437,  437,  437,  437,  437, 
          437,  437,  437,  437,  437,  437,    0,  520,  437,  437, 
          437,  437,    0,    0,  437,    0,    0,  437,    0,    0, 
          437,    0,  437,    0,  437,  519,  437,    0,  437,  437, 
          437,  437,  437,  437,  437,  519,  437,  437,  437,    0, 
            0,    0,   52,   52,   52,    0,    0,   52,   52,   52, 
          437,   52,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   52,    0,    0,    0,    0,    0,    0,    0,  519, 
          437,    0,    0,   52,   52,   52,   52,   52,    0,    0, 
            0,    0,    0,  519,  519,    0,  115,    0,  519,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  519,    0,    0,    0, 
            0,    0,   52,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  645,    0,    0,   96,    0,   98, 
            0,    0,  125,   52,  645,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   96,    0,   98,    0,   52,    0,    0,  520,  520, 
          520,    0,    0,  520,  520,  520,    0,  520,   96,    0, 
           98,  125,    0,  645,    0,    0,    0,  520,  520,    0, 
            0,    0,    0,    0,    0,    0,  520,  520,    0,  520, 
          520,  520,  520,  520,    0,    0,    0,    0,  519,    0, 
          521,  520,  521,  619,    0,    0,    0,    0,    0,    0, 
            0,    0,  521,  520,  520,  520,  520,  520,  520,  520, 
          520,  520,  520,  520,  520,  520,  520,    0,    0,  520, 
          520,  520,  520,    0,  532,  520,    0,    0,  520,    0, 
            0,  520,    0,  520,    0,  520,  521,  520,    0,  520, 
          520,  520,  520,  520,  520,  520,    0,  520,    0,  520, 
          521,  521,    0,  117,    0,  521,    0,  519,  519,  519, 
            0,  520,  519,  519,  519,    0,  519,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  519,  519,    0,    0, 
            0,  520,    0,  521,    0,  519,  519,    0,  519,  519, 
          519,  519,  519,    0,    0,    0,    0,    0,    0,    0, 
          519,  522,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  522,  519,  519,  519,  519,  519,  519,  519,  519, 
          519,  519,  519,  519,  519,  519,    0,    0,  519,  519, 
          519,  519,    0,  531,  519,    0,    0,  519,    0,    0, 
          519,    0,  519,    0,  519,  522,  519,    0,  519,  519, 
          519,  519,  519,  519,  519,    0,  519,    0,  519,  522, 
          522,    0,  118,    0,  522,    0,  645,  645,  645,  125, 
          519,  645,  645,  645,    0,  645,   96,    0,   98,    0, 
            0,    0,  619,    0,    0,  645,  645,    0,    0,    0, 
          519,    0,  522,    0,  645,  645,    0,  645,  645,  645, 
          645,  645,    0,    0,    0,    0,    0,    0,    0,    0, 
           60,   99,    0,    0,  101,    0,    0,    0,    0,    0, 
           60,    0,    0,    0,  100,  519,    0,  521,    0,    0, 
          619,    0,  519,  519,  521,  521,    0,  619,  619,   96, 
            0,   98,    0,  619,    0,   99,  645,    0,  101,  645, 
            0,  645,    0,    0,  521,  521,  521,    0,  100,  521, 
          521,  521,   99,  521,  619,  101,    0,  645,    0,   60, 
            0,    0,    0,  521,  521,  100,    0,    0,    0,    0, 
            0,    0,  521,  521,    0,  521,  521,  521,  521,  521, 
            0,    0,  522,    0,    0,  524,    0,  521,  524,  645, 
            0,    0,    0,    0,    0,  523,    0,    0,  524,  521, 
          521,  521,  521,  521,  521,  521,  521,  521,  521,  521, 
          521,  521,  521,    0,    0,  521,  521,  521,  521,    0, 
          533,  521,    0,    0,  521,    0,    0,  521,    0,  521, 
            0,  521,  524,  521,    0,  521,  521,  521,  521,  521, 
          521,  521,    0,  521,    0,  521,  524,  524,    0,  120, 
            0,  524,    0,  522,  522,  522,    0,  521,  522,  522, 
          522,    0,  522,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  522,  522,    0,    0,    0,  521,    0,  524, 
            0,  522,  522,    0,  522,  522,  522,  522,  522,    0, 
            0,    0,    0,    0,    0,    0,  522,  523,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  523,  522,  522, 
          522,  522,  522,  522,  522,  522,  522,  522,  522,  522, 
          522,  522,    0,    0,  522,  522,  522,  522,    0,  534, 
          522,    0,    0,  522,    0,    0,  522,    0,  522,    0, 
          522,  523,  522,    0,  522,  522,  522,  522,  522,  522, 
          522,    0,  522,    0,  522,  523,  523,    0,  119,    0, 
          523,    0,   60,   60,   60,    0,  522,   60,   60,   60, 
           99,   60,    0,  101,    0,    0,    0,    0,    0,    0, 
            0,   60,   60,  100,    0,    0,  522,    0,  523,    0, 
           60,   60,    0,   60,   60,   60,   60,   60,    0,    0, 
            0,    0,    0,    0,    0,    0,   64,  102,    0,   93, 
            0,    0,    0,    0,    0,    0,   64,    0,    0,  522, 
           92,    0,  524,    0,    0,    0,  522,  522,    0,  524, 
          524,    0,  523,   99,    0,    0,  101,    0,    0,  523, 
          523,  102,   60,   93,    0,   60,  100,    0,    0,    0, 
          524,  524,  524,    0,   92,  524,  524,  524,  102,  524, 
           93,    0,    0,   60,    0,   64,    0,    0,    0,  524, 
          524,   92,    0,    0,    0,    0,    0,    0,  524,  524, 
            0,  524,  524,  524,  524,  524,    0,    0,  525,    0, 
          516,    0,    0,  524,  525,   60,    0,    0,    0,    0, 
            0,  515,    0,    0,  525,  524,  524,  524,  524,  524, 
          524,  524,  524,  524,  524,  524,  524,  524,  524,    0, 
            0,  524,  524,  524,  524,    0,  536,  524,    0,    0, 
          524,    0,    0,  524,    0,  524,    0,  524,  525,  524, 
            0,  524,  524,  524,  524,  524,  524,  524,    0,  524, 
            0,  524,  525,  525,    0,  121,    0,  525,    0,  523, 
          523,  523,    0,  524,  523,  523,  523,    0,  523,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  523,  523, 
            0,    0,    0,  524,    0,  525,    0,  523,  523,    0, 
          523,  523,  523,  523,  523,    0,    0,    0,    0,    0, 
            0,    0,  523,  516,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  516,  523,  523,  523,  523,  523,  523, 
          523,  523,  523,  523,  523,  523,  523,  523,    0,    0, 
          523,  523,  523,  523,    0,  535,  523,    0,    0,  523, 
            0,    0,  523,    0,  523,    0,  523,  516,  523,    0, 
          523,  523,  523,  523,  523,  523,  523,    0,  523,    0, 
          523,  516,  516,    0,  112,    0,  516,    0,   64,   64, 
           64,    0,  523,   64,   64,   64,  102,   64,   93,    0, 
            0,    0,    0,    0,    0,    0,    0,   64,   64,   92, 
            0,    0,  523,    0,  516,    0,   64,   64,    0,   64, 
           64,   64,   64,   64,    0,    0,    0,    0,    0,    0, 
            0,    0,   62,   95,    0,  109,    0,    0,    0,    0, 
            0,    0,   62,    0,    0,  525,  108,  516,    0,    0, 
            0,    0,  525,  525,  516,  516,    0,    0,  515,  102, 
            0,   93,    0,    0,    0,  515,  515,   95,   64,  109, 
            0,   64,   92,    0,    0,    0,  525,  525,  525,    0, 
          108,  525,  525,  525,   95,  525,  109,    0,    0,   64, 
            0,   62,    0,    0,    0,  525,  525,  108,    0,    0, 
            0,    0,    0,    0,  525,  525,    0,  525,  525,  525, 
          525,  525,    0,    0,  518,    0,  306,    0,    0,  525, 
          515,   64,    0,    0,    0,    0,    0,  317,    0,    0, 
          515,  525,  525,  525,  525,  525,  525,  525,  525,  525, 
          525,  525,  525,  525,  525,    0,    0,  525,  525,  525, 
          525,    0,  537,  525,    0,    0,  525,    0,    0,  525, 
            0,  525,    0,  525,  515,  525,    0,  525,  525,  525, 
          525,  525,  525,  525,    0,  525,    0,  525,  515,  515, 
            0,  111,    0,  515,    0,  516,  516,  516,    0,  525, 
          516,  516,  516,    0,  516,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  516,  516,    0,    0,    0,  525, 
            0,  515,    0,  516,  516,    0,  516,  516,  516,  516, 
          516,    0,    0,    0,    0,    0,    0,    0,  516,  518, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  518, 
          516,  516,  516,  516,  516,  516,  516,  516,  516,  516, 
          516,  516,  516,  516,    0,    0,  516,  516,  516,  516, 
            0,  528,  516,    0,    0,  516,    0,    0,  516,    0, 
          516,    0,  516,  518,  516,    0,  516,  516,  516,  516, 
          516,  516,  516,    0,  516,    0,  516,  518,  518,    0, 
          114,    0,  518,    0,   62,   62,   62,    0,  516,   62, 
           62,   62,   95,   62,  109,    0,    0,    0,    0,    0, 
            0,    0,    0,   62,   62,  108,    0,    0,  516,    0, 
          518,    0,   62,   62,    0,   62,   62,   62,   62,   62, 
            0,    0,    0,    0,    0,    0,    0,    0,   51,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   51,    0, 
            0,  518,    0,  306,    0,    0,    0,    0,  518,  518, 
          306,  306,    0,    0,  317,   95,    0,  109,    0,    0, 
            0,  317,  317,    0,   62,    0,    0,   62,  108,    0, 
            0,    0,  515,  515,  515,    0,    0,  515,  515,  515, 
            0,  515,    0,    0,    0,   62,    0,   51,    0,    0, 
            0,  515,  515,    0,    0,    0,    0,    0,    0,    0, 
          515,  515,    0,  515,  515,  515,  515,  515,    0,    0, 
            0,    0,    0,    0,    0,  515,  644,   62,    0,    0, 
            0,    0,    0,    0,    0,    0,  644,  515,  515,  515, 
          515,  515,  515,  515,  515,  515,  515,  515,  515,  515, 
          515,    0,    0,  515,  515,  515,  515,    0,  527,  515, 
            0,    0,  515,    0,    0,  515,    0,  515,    0,  515, 
          644,  515,    0,  515,  515,  515,  515,  515,  515,  515, 
            0,  515,    0,  515,  644,  644,    0,    0,    0,  644, 
            0,  518,  518,  518,    0,  515,  518,  518,  518,    0, 
          518,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          518,  518,    0,    0,    0,  515,    0,  644,    0,  518, 
          518,    0,  518,  518,  518,  518,  518,    0,    0,    0, 
            0,    0,    0,    0,  518,  306,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  306,  518,  518,  518,  518, 
          518,  518,  518,  518,  518,  518,  518,  518,  518,  518, 
            0,    0,  518,  518,  518,  518,    0,  530,  518,    0, 
            0,  518,    0,    0,  518,    0,  518,    0,  518,  306, 
          518,    0,  518,  518,  518,  518,  518,  518,  518,    0, 
          518,    0,  518,  306,  306,    0,  128,    0,  306,    0, 
           51,   51,   51,    0,  518,   51,   51,   51,    0,   51, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   51, 
            0,    0,    0,    0,  518,    0,  306,    0,   51,   51, 
            0,   51,   51,   51,   51,   51,    0,    0,    0,    0, 
            0,    0,    0,    0,   53,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   53,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           51,    0,    0,    0,    0,    0,    0,    0,  644,  644, 
          644,    0,    0,  644,  644,  644,    0,  644,    0,    0, 
            0,   51,    0,   53,    0,    0,  644,  644,  644,    0, 
            0,    0,    0,    0,    0,    0,  644,  644,    0,  644, 
          644,  644,  644,  644,    0,    0,    0,    0,    0,    0, 
            0,  644,  317,   51,    0,    0,    0,    0,    0,    0, 
            0,    0,  317,  644,  644,  644,  644,  644,  644,  644, 
          644,  644,  644,  644,  644,  644,  644,    0,    0,  644, 
          644,  644,  644,    0,    0,  644,    0,    0,  644,    0, 
            0,  644,    0,  644,    0,  644,  317,  644,    0,  644, 
          644,  644,  644,  644,  644,  644,    0,  644,  644,  644, 
          317,  317,    0,  127,    0,  317,    0,  306,  306,  306, 
            0,  644,  306,  306,  306,    0,  306,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  306,  306,    0,    0, 
            0,  644,    0,  317,    0,  306,  306,    0,  306,  306, 
          306,  306,  306,    0,    0,    0,    0,    0,    0,    0, 
          306,  458,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  458,  306,  306,  306,  306,  306,  306,  306,  306, 
          306,  306,  306,  306,  306,  306,    0,    0,  306,  306, 
          306,  306,    0,    0,  306,    0,    0,  306,    0,    0, 
          306,    0,  306,    0,  306,  458,  306,    0,  306,  306, 
          306,  306,  306,  306,  306,    0,  306,    0,  306,  458, 
          458,    0,    0,    0,  458,    0,   53,   53,   53,    0, 
          306,   53,   53,   53,    0,   53,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,   53,    0,    0,    0,    0, 
          306,    0,  458,    0,   53,   53,    0,   53,   53,   53, 
           53,   53,    0,    0,    0,    0,    0,    0,    0,    0, 
           55,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           55,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   53,    0,    0,    0, 
            0,    0,    0,    0,  317,  317,  317,    0,    0,  317, 
          317,  317,    0,  317,    0,    0,    0,   53,    0,   55, 
            0,    0,    0,  317,  317,    0,    0,    0,    0,    0, 
            0,    0,  317,  317,    0,  317,  317,  317,  317,  317, 
            0,    0,    0,    0,    0,    0,    0,  317,  328,   53, 
            0,    0,    0,    0,    0,    0,    0,    0,  328,  317, 
          317,  317,  317,  317,  317,  317,  317,  317,  317,  317, 
          317,  317,  317,    0,    0,  317,  317,  317,  317,    0, 
            0,  317,    0,    0,  317,    0,    0,  317,    0,  317, 
            0,  317,  328,  317,    0,  317,  317,  317,  317,  317, 
          317,  317,    0,  317,    0,  317,  328,  328,    0,    0, 
            0,  328,    0,  458,  458,  458,    0,  317,  458,  458, 
          458,    0,  458,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  458,  458,    0,    0,    0,  317,    0,  328, 
            0,  458,  458,    0,  458,  458,  458,  458,  458,    0, 
            0,    0,    0,    0,    0,    0,  458,  264,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  264,  458,  458, 
          458,  458,  458,  458,  458,  458,  458,  458,  458,  458, 
          458,  458,    0,    0,  458,  458,  458,  458,    0,    0, 
          458,    0,    0,  458,    0,    0,  458,    0,  458,    0, 
          458,  264,  458,    0,  458,  458,  458,  458,  458,  458, 
          458,    0,  458,    0,  458,  264,  264,    0,    0,    0, 
          264,    0,   55,   55,   55,    0,  458,   55,   55,   55, 
            0,   55,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   55,    0,    0,    0,    0,  458,    0,  360,    0, 
           55,   55,    0,   55,   55,   55,   55,   55,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   55,    0,    0,    0,    0,    0,    0,    0, 
          328,  328,  328,    0,    0,  328,  328,  328,    0,  328, 
            0,    0,    0,   55,    0,    0,   43,    0,    0,  328, 
          328,    0,    0,    0,    0,    0,    0,    0,  328,  328, 
            0,  328,  328,  328,  328,  328,    0,    0,    0,    0, 
            0,    0,    0,  328,  323,   55,    0,    0,    0,    0, 
            0,    0,    0,    0,  323,  328,  328,  328,  328,  328, 
          328,  328,  328,  328,  328,  328,  328,  328,  328,    0, 
            0,  328,  328,  328,  328,    0,    0,  328,    0,    0, 
          328,    0,    0,  328,    0,  328,    0,  328,  323,  328, 
            0,  328,  328,  328,  328,  328,  328,  328,    0,  328, 
            0,  328,  323,  323,    0,    0,    0,  323,    0,  264, 
          264,  264,    0,  328,  264,  264,  264,    0,  264,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  264,  264, 
            0,    0,    0,  328,    0,  323,    0,  264,  264,    0, 
          264,  264,  264,  264,  264,    0,    0,    0,    0,    0, 
            0,    0,  264,  509,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  509,  264,  264,  264,  264,  264,  264, 
          264,  264,  264,  264,  264,  360,  264,  264,    0,    0, 
          264,  264,  360,  360,    0,    0,  264,    0,    0,  264, 
            0,    0,  264,    0,  264,    0,  264,  509,  264,    0, 
          264,  264,  264,  264,  264,  264,  264,    0,  264,    0, 
          264,  509,  509,    0,    0,    0,  509,    0,    0,   43, 
           43,   43,  264,    0,    0,   43,   43,    0,   43,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  264,    0,  509,    0,    0,    0,    0,    0, 
           43,   43,   43,   43,   43,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  290,    0,    0,    0,    0,    0,    0,   43, 
            0,    0,  290,    0,    0,    0,  323,  323,  323,    0, 
            0,  323,  323,  323,    0,  323,    0,    0,    0,    0, 
           43,    0,    0,    0,    0,  323,  323,    0,    0,    0, 
            0,    0,    0,    0,  323,  323,  290,  323,  323,  323, 
          323,  323,    0,    0,    0,    0,    0,    0,    0,  323, 
          356,  290,   43,    0,    0,    0,    0,    0,    0,    0, 
          356,  323,  323,  323,  323,  323,  323,  323,  323,  323, 
          323,  323,  323,  323,  323,    0,    0,  323,  323,  323, 
          323,    0,    0,  323,    0,    0,  323,    0,    0,  323, 
            0,  323,    0,  323,  356,  323,    0,  323,  323,  323, 
          323,  323,  323,  323,    0,  323,    0,  323,    0,  356, 
            0,    0,    0,  356,    0,  509,  509,  509,    0,  323, 
          509,  509,  509,    0,  509,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  509,  509,    0,    0,    0,  323, 
            0,  356,    0,  509,  509,    0,  509,  509,  509,  509, 
          509,    0,    0,    0,    0,    0,    0,    0,  509,  357, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  357, 
            0,  509,  509,  509,  509,  509,  509,  509,  509,  509, 
          509,  509,  509,  509,    0,    0,  509,  509,  509,  509, 
            0,    0,  509,    0,    0,  509,    0,    0,  509,    0, 
          509,    0,  509,  357,  509,    0,  509,  509,  509,  509, 
          509,  509,  509,    0,  509,    0,  509,    0,  357,    0, 
            0,    0,  357,    0,    0,    0,    0,    0,  509,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  509,    0, 
          357,    0,    0,    0,  290,  290,  290,    0,    0,  290, 
          290,  290,    0,  290,    0,  320,    0,    0,    0,    0, 
            0,    0,    0,  290,  290,  320,    0,    0,    0,    0, 
            0,    0,  290,  290,    0,  290,  290,  290,  290,  290, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  320, 
            0,    0,  356,  356,  356,    0,    0,  356,  356,  356, 
            0,  356,    0,    0,  320,    0,    0,    0,  320,    0, 
            0,  356,  356,    0,  290,    0,    0,  290,    0,  290, 
          356,  356,    0,  356,  356,  356,  356,  356,    0,    0, 
            0,    0,    0,    0,    0,  290,  320,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  356,  356,  356, 
          356,  356,  356,  356,  356,  356,  356,  356,  356,  356, 
          356,    0,    0,  356,  356,  356,  356,  290,  242,  356, 
            0,    0,  356,    0,    0,  356,    0,  356,  242,  356, 
            0,  356,    0,  356,  356,  356,  356,  356,  356,  356, 
            0,  356,    0,  356,    0,    0,    0,    0,    0,    0, 
            0,  357,  357,  357,    0,    0,  357,  357,  357,    0, 
          357,    0,  242,    0,    0,    0,    0,    0,    0,    0, 
          357,  357,    0,    0,    0,  356,  242,  242,    0,  357, 
          357,  242,  357,  357,  357,  357,  357,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  357,  357,  357,  357, 
          357,  357,  357,  357,  357,  357,  357,  357,  357,  357, 
            0,    0,  357,  357,  357,  357,    0,    0,  357,    0, 
            0,  357,    0,    0,  357,    0,  357,    0,  357,  239, 
          357,    0,  357,  357,  357,  357,  357,  357,  357,  239, 
          357,    0,  357,    0,    0,    0,    0,  320,  320,  320, 
            0,    0,  320,  320,  320,    0,  320,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  320,  320,    0,    0, 
            0,    0,    0,  239,  357,  320,  320,    0,  320,  320, 
          320,  320,  320,    0,    0,    0,    0,  239,  239,    0, 
            0,    0,  239,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  320,  320,  320,  320,  320,  320,  320,  320, 
          320,  320,  320,  320,  320,  320,    0,    0,  320,  320, 
          320,  320,    0,    0,  320,    0,    0,  320,    0,    0, 
          320,    0,  320,    0,  320,    0,  320,    0,  320,  320, 
          320,  320,  320,  320,  320,    0,  320,    0,  320,  236, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  236, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          242,  242,  242,    0,    0,  242,  242,  242,    0,  242, 
          320,    0,    0,    0,    0,    0,    0,    0,    0,  242, 
          242,    0,    0,  236,    0,    0,    0,    0,  242,  242, 
            0,  242,  242,  242,  242,  242,    0,  236,  236,    0, 
            0,    0,  236,  242,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  242,  242,  242,  242, 
          242,  242,  242,  242,  242,  242,    0,  242,  242,    0, 
            0,  242,  242,    0,    0,    0,    0,  242,    0,    0, 
          242,    0,    0,  242,    0,  242,    0,  242,  238,  242, 
            0,  242,  242,  242,  242,  242,  242,  242,  238,  242, 
            0,  242,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  242,    0,    0,    0,    0,    0,    0, 
            0,  239,  239,  239,    0,    0,  239,  239,  239,    0, 
          239,    0,  238,  242,    0,    0,    0,    0,    0,    0, 
          239,  239,    0,    0,    0,    0,  238,  238,    0,  239, 
          239,  238,  239,  239,  239,  239,  239,    0,    0,    0, 
            0,    0,    0,    0,  239,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  239,  239,  239, 
          239,  239,  239,  239,  239,  239,  239,    0,  239,  239, 
            0,    0,  239,  239,    0,    0,    0,    0,  239,    0, 
            0,  239,    0,    0,  239,    0,  239,    0,  239,    0, 
          239,    0,  239,  239,  239,  239,  239,  239,  239,    0, 
          239,    0,  239,  237,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  237,  239,    0,    0,    0,    0,    0, 
            0,  236,  236,  236,    0,    0,  236,  236,  236,    0, 
          236,    0,    0,    0,  239,    0,    0,    0,    0,    0, 
          236,  236,    0,    0,    0,    0,    0,  237,    0,  236, 
          236,    0,  236,  236,  236,  236,  236,    0,    0,    0, 
            0,  237,  237,    0,  236,    0,  237,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  236,  236,  236, 
          236,  236,  236,  236,  236,  236,  236,    0,  236,  236, 
            0,    0,  236,  236,    0,    0,    0,    0,  236,    0, 
            0,  236,    0,    0,  236,    0,  236,    0,  236,    0, 
          236,    0,  236,  236,  236,  236,  236,  236,  236,    0, 
          236,    0,  236,    0,    0,    0,    0,    0,    0,    0, 
          238,  238,  238,    0,  236,  238,  238,  238,    0,  238, 
            0,  240,    0,    0,    0,    0,    0,    0,    0,  238, 
          238,  240,    0,    0,  236,    0,    0,    0,  238,  238, 
            0,  238,  238,  238,  238,  238,    0,    0,    0,    0, 
            0,    0,    0,  238,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  240,  238,  238,  238,  238, 
          238,  238,  238,  238,  238,  238,    0,  238,  238,  240, 
          240,  238,  238,    0,  240,    0,    0,  238,    0,    0, 
          238,    0,    0,  238,    0,  238,    0,  238,    0,  238, 
            0,  238,  238,  238,  238,  238,  238,  238,    0,  238, 
            0,  238,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  238,  438,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  438,    0,    0,    0,    0,    0, 
            0,    0,    0,  238,    0,  237,  237,  237,    0,    0, 
          237,  237,  237,    0,  237,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  237,  237,    0,    0,  103,    0, 
            0,    0,    0,  237,  237,    0,  237,  237,  237,  237, 
          237,    0,    0,  438,    0,  122,    0,  438,  237,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  237,  237,  237,  237,  237,  237,  237,  237,  237, 
          237,    0,  237,  237,    0,  438,  237,  237,    0,    0, 
            0,    0,  237,    0,    0,  237,    0,    0,  237,    0, 
          237,    0,  237,    0,  237,    0,  237,  237,  237,  237, 
          237,  237,  237,    0,  237,    0,  237,  520,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  520,  237,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  237,    0, 
            0,    0,    0,  240,  240,  240,    0,    0,  240,  240, 
          240,   97,  240,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  240,  240,    0,    0,  520,    0,  116,    0, 
          520,  240,  240,    0,  240,  240,  240,  240,  240,    0, 
            0,    0,    0,    0,    0,    0,  240,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  520,  240, 
          240,  240,  240,  240,  240,  240,  240,  240,  240,    0, 
          240,  240,    0,    0,  240,  240,    0,    0,    0,    0, 
          240,    0,    0,  240,    0,    0,  240,    0,  240,    0, 
          240,    0,  240,    0,  240,  240,  240,  240,  240,  240, 
          240,    0,  240,    0,  240,    0,  438,  438,  438,    0, 
            0,    0,  438,  438,    0,  438,  240,    0,    0,    0, 
            0,    0,    0,    0,  438,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  438,  438,  240,  438,  438,  438, 
          438,  438,    0,    0,    0,    0,    0,    0,    0,    0, 
          519,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          519,  438,  438,  438,  438,  438,  438,  438,  438,  438, 
          438,  438,  438,  438,  438,    0,    0,  438,  438,  438, 
          438,    0,    0,    0,    0,    0,  438,    0,    0,    0, 
            0,    0,    0,  438,   96,  438,    0,  438,  438,  438, 
          438,  438,  438,  438,    0,  438,  438,  438,    0,  519, 
            0,  115,    0,  519,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  520, 
          520,  520,    0,    0,    0,  520,  520,    0,  520,  438, 
            0,  519,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  520,  520,    0, 
          520,  520,  520,  520,  520,    0,    0,    0,  521,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  521,    0, 
            0,    0,    0,    0,  520,  520,  520,  520,  520,  520, 
          520,  520,  520,  520,  520,  520,  520,  520,    0,    0, 
          520,  520,  520,  520,    0,  532,    0,    0,    0,  520, 
            0,    0,   98,    0,    0,    0,  520,    0,  520,    0, 
          520,  520,  520,  520,  520,  520,  520,  521,  520,  117, 
          520,  521,    0,    0,    0,    0,    0,    0,    0,    0, 
          642,  600,    0,    0,  643,    0,    0,    0,  187,  188, 
            0,  189,  190,  191,  192,  193,  194,  195,    0,  521, 
          196,  197,  520,    0,    0,  198,  199,  200,  201,    0, 
            0,  522,    0,    0,    0,  303,    0,    0,    0,    0, 
            0,  522,  203,  204,    0,  205,  206,  207,  208,  209, 
          210,  211,  212,  213,  214,  215,    0,    0,  216,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  519,  519,  519,   99,    0,    0,  519,  519, 
            0,  519,    0,    0,    0,    0,    0,    0,    0,    0, 
          522,    0,  118,    0,  522,    0,    0,    0,    0,    0, 
          519,  519,    0,  519,  519,  519,  519,  519,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  522,    0,    0,    0,    0,  519,  519,  519, 
          519,  519,  519,  519,  519,  519,  519,  519,  519,  519, 
          519,  524,    0,  519,  519,  519,  519,    0,  531,    0, 
            0,  524,  519,    0,    0,    0,    0,    0,    0,  519, 
            0,  519,    0,  519,  519,  519,  519,  519,  519,  519, 
            0,  519,    0,  519,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  101,    0,    0,    0,    0, 
          521,  521,  521,    0,    0,    0,  521,  521,    0,  521, 
          524,    0,  120,    0,  524,  519,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  521,  521, 
            0,  521,  521,  521,  521,  521,    0,    0,    0,    0, 
            0,    0,  524,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  521,  521,  521,  521,  521, 
          521,  521,  521,  521,  521,  521,  521,  521,  521,    0, 
            0,  521,  521,  521,  521,    0,  533,    0,    0,    0, 
          521,    0,    0,    0,    0,    0,    0,  521,    0,  521, 
            0,  521,  521,  521,  521,  521,  521,  521,    0,  521, 
            0,  521,    0,  522,  522,  522,    0,    0,    0,  522, 
          522,    0,  522,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  522,  522,  521,  522,  522,  522,  522,  522,    0, 
            0,    0,  523,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  523,    0,    0,    0,    0,    0,  522,  522, 
          522,  522,  522,  522,  522,  522,  522,  522,  522,  522, 
          522,  522,    0,    0,  522,  522,  522,  522,    0,  534, 
            0,    0,    0,  522,    0,    0,  100,    0,    0,    0, 
          522,    0,  522,    0,  522,  522,  522,  522,  522,  522, 
          522,  523,  522,  119,  522,  523,    0,    0,    0,    0, 
            0,    0,    0,  524,  524,  524,    0,    0,    0,  524, 
          524,    0,  524,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  523,    0,    0,  522,    0,    0,    0, 
            0,  524,  524,    0,  524,  524,  524,  524,  524,    0, 
            0,    0,  525,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  525,    0,    0,    0,    0,    0,  524,  524, 
          524,  524,  524,  524,  524,  524,  524,  524,  524,  524, 
          524,  524,    0,    0,  524,  524,  524,  524,    0,  536, 
            0,    0,    0,  524,    0,    0,  102,    0,    0,    0, 
          524,    0,  524,    0,  524,  524,  524,  524,  524,  524, 
          524,  525,  524,  121,  524,  525,    0,    0,    0,    0, 
            0,    0,    0,    0,  613,  608,    0,    0,  614,    0, 
            0,    0,  187,  188,    0,  189,  190,  191,  192,  193, 
          194,  195,    0,  525,  196,  197,  524,    0,    0,  198, 
      };
   }

   private static final short[] yyTable3() {
      return new short[] {

          199,  200,  201,    0,    0,  516,    0,    0,    0,  303, 
            0,    0,    0,    0,    0,  516,  203,  204,    0,  205, 
          206,  207,  208,  209,  210,  211,  212,  213,  214,  215, 
            0,    0,  216,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   93, 
            0,    0,    0,    0,  523,  523,  523,    0,    0,    0, 
          523,  523,    0,  523,  516,    0,  112,    0,  516,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  523,  523,    0,  523,  523,  523,  523,  523, 
            0,    0,    0,    0,    0,    0,  516,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  523, 
          523,  523,  523,  523,  523,  523,  523,  523,  523,  523, 
          523,  523,  523,  515,    0,  523,  523,  523,  523,    0, 
          535,    0,    0,  515,  523,    0,    0,    0,    0,    0, 
            0,  523,    0,  523,    0,  523,  523,  523,  523,  523, 
          523,  523,    0,  523,    0,  523,    0,    0,    0,    0, 
            0,    0,    0,    0,  525,  525,  525,   92,    0,    0, 
          525,  525,    0,  525,    0,    0,    0,    0,    0,    0, 
            0,    0,  515,    0,  111,    0,  515,  523,    0,    0, 
            0,    0,  525,  525,    0,  525,  525,  525,  525,  525, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  515,    0,    0,    0,    0,  525, 
          525,  525,  525,  525,  525,  525,  525,  525,  525,  525, 
          525,  525,  525,    0,    0,  525,  525,  525,  525,    0, 
          537,    0,    0,    0,  525,    0,    0,    0,    0,    0, 
            0,  525,    0,  525,    0,  525,  525,  525,  525,  525, 
          525,  525,    0,  525,    0,  525,    0,  516,  516,  516, 
            0,    0,    0,  516,  516,    0,  516,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  516,  516,  525,  516,  516, 
          516,  516,  516,    0,    0,    0,  518,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  518,    0,    0,    0, 
            0,    0,  516,  516,  516,  516,  516,  516,  516,  516, 
          516,  516,  516,  516,  516,  516,    0,    0,  516,  516, 
          516,  516,    0,  528,    0,    0,    0,  516,    0,    0, 
           95,    0,    0,    0,  516,    0,  516,    0,  516,  516, 
          516,  516,  516,  516,  516,  518,  516,  114,  516,  518, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  515,  515,  515,    0,    0, 
            0,  515,  515,    0,  515,    0,    0,  518,    0,    0, 
          516,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  515,  515,    0,  515,  515,  515,  515, 
          515,    0,    0,    0,  306,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  306,    0,    0,    0,    0,    0, 
          515,  515,  515,  515,  515,  515,  515,  515,  515,  515, 
          515,  515,  515,  515,    0,    0,  515,  515,  515,  515, 
            0,  527,    0,    0,    0,  515,    0,    0,  109,    0, 
            0,    0,  515,    0,  515,    0,  515,  515,  515,  515, 
          515,  515,  515,  306,  515,  128,  515,  306,    0,    0, 
            0,    0,    0,    0,    0,    0,  690,  600,    0,    0, 
          691,    0,    0,    0,  187,  188,    0,  189,  190,  191, 
          192,  193,  194,  195,    0,  306,  196,  197,  515,    0, 
            0,  198,  199,  200,  201,    0,    0,  317,    0,    0, 
            0,  303,    0,    0,    0,    0,    0,  317,  203,  204, 
            0,  205,  206,  207,  208,  209,  210,  211,  212,  213, 
          214,  215,    0,    0,  216,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  518,  518, 
          518,  108,    0,    0,  518,  518,    0,  518,    0,    0, 
            0,    0,    0,    0,    0,    0,  317,    0,  127,    0, 
          317,    0,    0,    0,    0,    0,  518,  518,    0,  518, 
          518,  518,  518,  518,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  317,    0, 
            0,    0,    0,  518,  518,  518,  518,  518,  518,  518, 
          518,  518,  518,  518,  518,  518,  518,    0,    0,  518, 
          518,  518,  518,    0,  530,    0,  234,    0,  518,    0, 
            0,    0,    0,    0,    0,  518,  234,  518,    0,  518, 
          518,  518,  518,  518,  518,  518,    0,  518,    0,  518, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  306,  306,  306,    0, 
          234,    0,  306,  306,    0,  306,    0,    0,    0,    0, 
            0,  518,    0,    0,  234,  234,    0,    0,    0,  234, 
            0,    0,    0,    0,  306,  306,    0,  306,  306,  306, 
          306,  306,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  306,  306,  306,  306,  306,  306,  306,  306,  306, 
          306,  306,  306,  306,  306,    0,    0,  306,  306,  306, 
          306,    0,    0,    0,    0,    0,  306,    0,    0,    0, 
            0,    0,    0,  306,    0,  306,    0,  306,  306,  306, 
          306,  306,  306,  306,    0,  306,    0,  306,    0,  317, 
          317,  317,    0,    0,    0,  317,  317,    0,  317,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  317,  317,  306, 
          317,  317,  317,  317,  317,    0,    0,    0,    0,    0, 
            0,    0,    0,  235,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  235,  317,  317,  317,  317,  317,  317, 
          317,  317,  317,  317,  317,  317,  317,  317,    0,    0, 
          317,  317,  317,  317,    0,    0,    0,    0,    0,  317, 
            0,    0,    0,    0,    0,    0,  317,  235,  317,    0, 
          317,  317,  317,  317,  317,  317,  317,    0,  317,    0, 
          317,  235,  235,    0,    0,    0,  235,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  234,  234, 
          234,    0,    0,  234,  234,  234,    0,  234,    0,    0, 
            0,    0,  317,    0,    0,    0,    0,  234,  234,    0, 
            0,    0,    0,    0,    0,    0,  234,  234,    0,  234, 
          234,  234,  234,  234,    0,    0,    0,    0,    0,    0, 
            0,  234,  258,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  258,    0,  234,  234,  234,  234,  234,  234, 
          234,  234,  234,  234,    0,  234,  234,    0,    0,  234, 
          234,    0,    0,    0,    0,  234,    0,    0,  234,    0, 
            0,  234,    0,  234,    0,    0,  258,  234,    0,    0, 
            0,  234,  234,  234,  234,  234,    0,  234,    0,  234, 
          258,  258,    0,    0,    0,  258,    0,    0,    0,    0, 
            0,  234,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  693,  608,    0,    0,  694,    0,    0,    0,  187, 
          188,  234,  189,  190,  191,  192,  193,  194,  195,    0, 
            0,  196,  197,    0,    0,    0,  198,  199,  200,  201, 
            0,    0,  259,    0,    0,    0,  303,    0,    0,    0, 
            0,    0,  259,  203,  204,    0,  205,  206,  207,  208, 
          209,  210,  211,  212,  213,  214,  215,    0,    0,  216, 
            0,    0,    0,    0,    0,  235,  235,  235,    0,    0, 
          235,  235,  235,    0,  235,    0,  259,    0,    0,    0, 
            0,    0,    0,    0,  235,  235,    0,    0,    0,    0, 
          259,  259,    0,  235,  235,  259,  235,  235,  235,  235, 
          235,    0,    0,    0,    0,    0,    0,    0,  235,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  235,  235,  235,  235,  235,  235,  235,  235,  235, 
          235,    0,  235,  235,    0,    0,  235,  235,    0,    0, 
            0,    0,  235,    0,    0,  235,    0,    0,  235,    0, 
          235,    0,    0,    0,  235,    0,    0,    0,  235,  235, 
          235,  235,  235,    0,  235,    0,  235,  245,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  245,  235,    0, 
            0,    0,    0,    0,  258,  258,  258,    0,    0,  258, 
          258,  258,    0,  258,    0,    0,    0,    0,  235,    0, 
            0,    0,    0,  258,  258,    0,    0,    0,    0,    0, 
            0,  245,  258,  258,    0,  258,  258,  258,  258,  258, 
            0,    0,    0,    0,    0,  245,  245,  258,    0,    0, 
          245,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          258,  258,  258,  258,  258,  258,  258,  258,  258,  258, 
            0,  258,  258,    0,    0,  258,  258,    0,    0,    0, 
            0,  258,    0,    0,  258,    0,    0,  258,    0,  258, 
            0,    0,    0,  258,    0,    0,  243,    0,    0,  258, 
          258,  258,    0,  258,    0,  258,  243,    0,    0,    0, 
            0,    0,    0,    0,  259,  259,  259,  258,    0,  259, 
          259,  259,    0,  259,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  259,  259,    0,    0,  258,    0,    0, 
          243,    0,  259,  259,    0,  259,  259,  259,  259,  259, 
            0,    0,    0,    0,  243,  243,    0,  259,    0,  243, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          259,  259,  259,  259,  259,  259,  259,  259,  259,  259, 
            0,  259,  259,    0,    0,  259,  259,    0,    0,    0, 
            0,  259,    0,    0,  259,    0,    0,  259,    0,  259, 
            0,    0,    0,  259,    0,    0,    0,    0,    0,  259, 
          259,  259,    0,  259,    0,  259,  244,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  244,  259,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  259,    0,  245, 
          245,  245,    0,    0,  245,  245,  245,    0,  245,    0, 
          244,    0,    0,    0,    0,    0,    0,    0,  245,  245, 
            0,    0,    0,    0,  244,  244,    0,  245,  245,  244, 
          245,  245,  245,  245,  245,    0,    0,    0,    0,    0, 
            0,    0,  245,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  245,  245,  245,  245,  245, 
          245,  245,  245,  245,  245,    0,  245,  245,    0,    0, 
            0,    0,    0,    0,    0,  248,  245,    0,    0,  245, 
            0,    0,  245,    0,  245,  248,    0,    0,  245,    0, 
            0,    0,    0,    0,  245,  245,  245,    0,  245,    0, 
          245,    0,    0,    0,    0,    0,    0,    0,  243,  243, 
          243,    0,  245,  243,  243,  243,    0,  243,    0,  248, 
            0,    0,    0,    0,    0,    0,    0,  243,  243,    0, 
            0,    0,  245,  248,  248,    0,  243,  243,  248,  243, 
          243,  243,  243,  243,    0,    0,    0,    0,    0,    0, 
            0,  243,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  243,  243,  243,  243,  243,  243, 
          243,  243,  243,  243,    0,  243,  243,    0,    0,    0, 
            0,    0,    0,    0,    0,  243,    0,    0,  243,    0, 
            0,  243,  250,  243,    0,    0,    0,    0,    0,    0, 
            0,    0,  250,  243,  243,  243,    0,  243,    0,  243, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  243,    0,    0,    0,    0,    0,    0,  244,  244, 
          244,    0,    0,  244,  244,  244,  250,  244,    0,    0, 
            0,  243,    0,    0,    0,    0,    0,  244,  244,    0, 
          250,  250,    0,    0,    0,  250,  244,  244,    0,  244, 
          244,  244,  244,  244,    0,    0,    0,    0,    0,    0, 
            0,  244,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  244,  244,  244,  244,  244,  244, 
          244,  244,  244,  244,    0,  244,  244,    0,    0,    0, 
            0,    0,    0,    0,    0,  244,    0,    0,  244,    0, 
            0,  244,    0,  244,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  244,  244,  244,    0,  244,    0,  244, 
            0,    0,    0,    0,    0,    0,    0,  248,  248,  248, 
            0,  244,  248,  248,  248,    0,  248,    0,  249,    0, 
            0,    0,    0,    0,    0,    0,  248,  248,  249,    0, 
            0,  244,    0,    0,    0,  248,  248,    0,  248,  248, 
          248,  248,  248,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  249,  248,  248,  248,  248,  248,  248,  248, 
          248,  248,  248,    0,  248,  248,  249,  249,    0,    0, 
            0,  249,    0,    0,  248,    0,    0,  248,    0,    0, 
          248,    0,  248,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  248,  248,    0,    0,    0,    0,  248,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,    0,  250,  250,  250,    0,    0,  250, 
          250,  250,    0,  250,    0,  247,    0,    0,    0,    0, 
          248,    0,    0,  250,  250,  247,    0,    0,    0,    0, 
            0,    0,  250,  250,    0,  250,  250,  250,  250,  250, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  247, 
          250,  250,  250,  250,  250,  250,  250,  250,  250,  250, 
            0,  250,  250,  247,  247,    0,    0,    0,  247,    0, 
            0,  250,    0,    0,  250,    0,    0,  250,    0,  250, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  250, 
          250,    0,    0,    0,    0,  250,    0,    0,    0,    0, 
            0,    0,  251,    0,    0,    0,    0,  250,    0,    0, 
            0,    0,  251,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  251,    0,    0,    0, 
          249,  249,  249,    0,    0,  249,  249,  249,    0,  249, 
          251,  251,    0,    0,    0,  251,    0,    0,    0,  249, 
          249,    0,    0,    0,    0,    0,    0,    0,  249,  249, 
            0,  249,  249,  249,  249,  249,    0,    0,    0,    0, 
          252,    0,    0,  249,    0,    0,    0,    0,    0,    0, 
          252,    0,    0,    0,    0,    0,  249,  249,  249,  249, 
          249,  249,  249,  249,  249,  249,    0,  249,  249,    0, 
            0,    0,    0,    0,    0,    0,    0,  249,    0,    0, 
          249,    0,    0,  249,  252,  249,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  249,  249,    0,  252,  252, 
            0,  249,    0,  252,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  249,    0,    0,    0,  247,  247,  247, 
            0,    0,  247,  247,  247,    0,  247,    0,  253,    0, 
            0,    0,    0,  249,    0,    0,  247,  247,  253,    0, 
            0,    0,    0,    0,    0,  247,  247,    0,  247,  247, 
          247,  247,  247,    0,    0,    0,    0,    0,    0,    0, 
          247,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  253,  247,  247,  247,  247,  247,  247,  247, 
          247,  247,  247,    0,  247,  247,  253,  253,    0,    0, 
            0,  253,    0,    0,  247,    0,    0,  247,    0,    0, 
          247,    0,  247,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  247,  247,  251,  251,  251,    0,  247,  251, 
          251,  251,    0,  251,    0,  260,    0,    0,    0,    0, 
          247,    0,    0,  251,  251,  260,    0,    0,    0,    0, 
            0,    0,  251,  251,    0,  251,  251,  251,  251,  251, 
          247,    0,    0,    0,    0,    0,    0,  251,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  260, 
            0,    0,    0,    0,    0,    0,  251,  251,    0,    0, 
            0,  251,  251,  260,  260,    0,    0,    0,  260,    0, 
            0,  251,    0,    0,  251,    0,    0,  251,    0,  251, 
            0,    0,  252,  252,  252,    0,    0,  252,  252,  252, 
            0,  252,    0,  254,    0,  251,    0,    0,    0,    0, 
            0,  252,  252,  254,    0,    0,    0,  251,    0,    0, 
          252,  252,    0,  252,  252,  252,  252,  252,    0,    0, 
            0,    0,    0,    0,    0,  252,    0,  251,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  254,    0,    0, 
            0,    0,    0,    0,  252,  252,    0,    0,    0,  252, 
          252,  254,  254,    0,    0,    0,  254,    0,    0,  252, 
            0,    0,  252,    0,    0,  252,    0,  252,    0,    0, 
          253,  253,  253,    0,    0,  253,  253,  253,    0,  253, 
            0,  255,    0,  252,    0,    0,    0,    0,    0,  253, 
          253,  255,    0,    0,    0,  252,    0,    0,  253,  253, 
            0,  253,  253,  253,  253,  253,    0,    0,    0,    0, 
            0,    0,    0,  253,    0,  252,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  255,    0,    0,    0,    0, 
            0,    0,  253,  253,    0,    0,    0,  253,  253,  255, 
          255,    0,    0,    0,  255,    0,    0,  253,    0,    0, 
          253,    0,    0,  253,    0,  253,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  260,  260,  260, 
            0,  253,  260,  260,  260,    0,  260,    0,  261,    0, 
            0,    0,    0,  253,    0,    0,  260,  260,  261,    0, 
            0,    0,    0,    0,    0,  260,  260,    0,  260,  260, 
          260,  260,  260,  253,    0,    0,    0,    0,    0,    0, 
          260,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  261,    0,    0,    0,    0,    0,    0,  260, 
          260,    0,    0,    0,  260,  260,  261,  261,    0,    0, 
            0,  261,    0,    0,  260,    0,    0,  260,    0,    0, 
          260,    0,  260,    0,    0,  254,  254,  254,    0,    0, 
          254,  254,  254,    0,  254,    0,  232,    0,  260,    0, 
            0,    0,    0,    0,  254,  254,  232,    0,    0,    0, 
          260,    0,    0,  254,  254,    0,  254,  254,  254,  254, 
          254,    0,    0,    0,    0,    0,    0,    0,  254,    0, 
          260,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          232,    0,    0,    0,    0,    0,    0,  254,  254,    0, 
            0,    0,  254,  254,  232,  232,    0,    0,    0,  232, 
            0,    0,  254,    0,    0,  254,    0,    0,  254,    0, 
          254,    0,    0,  255,  255,  255,    0,    0,  255,  255, 
          255,    0,  255,    0,  233,    0,  254,    0,    0,    0, 
            0,    0,  255,  255,  233,    0,    0,    0,  254,    0, 
            0,  255,  255,    0,  255,  255,  255,  255,  255,    0, 
            0,    0,    0,    0,    0,    0,  255,    0,  254,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  233,    0, 
            0,    0,    0,    0,    0,  255,  255,    0,    0,    0, 
          255,  255,  233,  233,    0,    0,    0,  233,    0,    0, 
          255,    0,    0,  255,    0,    0,  255,    0,  255,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          261,  261,  261,  231,  255,  261,  261,  261,    0,  261, 
            0,    0,    0,  231,    0,    0,  255,    0,    0,  261, 
          261,    0,    0,    0,    0,    0,    0,    0,  261,  261, 
            0,  261,  261,  261,  261,  261,  255,    0,    0,    0, 
            0,    0,    0,  261,    0,    0,    0,  231,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  231,  231,  261,    0,    0,    0,  261,  261,    0, 
            0,    0,    0,    0,    0,    0,    0,  261,    0,    0, 
          261,    0,    0,  261,    0,  261,    0,    0,  232,  232, 
          232,    0,  230,  232,  232,  232,    0,  232,    0,    0, 
            0,  261,  230,    0,    0,    0,    0,  232,  232,    0, 
            0,    0,    0,  261,    0,    0,  232,  232,    0,  232, 
          232,  232,  232,  232,    0,    0,    0,    0,    0,    0, 
            0,  232,    0,  261,    0,    0,  230,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          230,  230,    0,    0,  228,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  228,  232,    0,    0,  232,    0, 
            0,  232,    0,  232,    0,    0,  233,  233,  233,    0, 
            0,  233,  233,  233,    0,  233,    0,    0,    0,  232, 
            0,    0,    0,    0,    0,  233,  233,    0,  228,    0, 
            0,  232,    0,    0,  233,  233,    0,  233,  233,  233, 
          233,  233,  228,  228,    0,    0,    0,    0,    0,  233, 
            0,  232,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  233,    0,    0,  233,    0,    0,  233, 
            0,  233,    0,  229,    0,  231,  231,  231,    0,    0, 
          231,  231,  231,  229,  231,    0,    0,  233,    0,    0, 
            0,    0,    0,    0,  231,  231,    0,    0,    0,  233, 
            0,    0,    0,  231,  231,    0,  231,  231,  231,  231, 
          231,    0,    0,    0,    0,    0,    0,  229,  231,  233, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  229,  229,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  231,    0,    0,  231,    0,    0,  231,    0, 
          231,    0,  226,    0,  230,  230,  230,    0,    0,  230, 
          230,  230,  226,  230,    0,    0,  231,    0,    0,    0, 
            0,    0,    0,  230,  230,    0,    0,    0,  231,    0, 
            0,    0,  230,  230,    0,  230,  230,  230,  230,  230, 
            0,    0,    0,    0,    0,    0,  226,  230,  231,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          226,  226,    0,    0,  227,    0,  228,  228,  228,    0, 
            0,  228,  228,  228,  227,  228,    0,    0,    0,    0, 
            0,  230,    0,    0,  230,  228,  228,  230,    0,  230, 
            0,    0,    0,    0,  228,  228,    0,  228,  228,  228, 
          228,  228,    0,    0,    0,  230,    0,    0,  227,  228, 
            0,    0,    0,    0,    0,    0,    0,  230,    0,    0, 
            0,    0,  227,  227,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  230,    0,    0, 
            0,    0,    0,  228,    0,    0,  228,    0,    0,  228, 
            0,  228,    0,  222,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  222,    0,    0,    0,  228,    0,    0, 
            0,    0,    0,    0,    0,  229,  229,  229,    0,  228, 
          229,  229,  229,    0,  229,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  229,  229,    0,  222,    0,  228, 
            0,    0,    0,  229,  229,    0,  229,  229,  229,  229, 
          229,  222,  222,    0,    0,    0,    0,    0,  229,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  229,    0,    0,  229,    0,    0,  229,    0, 
          229,    0,  224,    0,  226,  226,  226,    0,    0,  226, 
          226,  226,  224,  226,    0,    0,  229,    0,    0,    0, 
            0,    0,    0,  226,  226,    0,    0,    0,  229,    0, 
            0,    0,  226,  226,    0,  226,  226,  226,  226,  226, 
            0,    0,    0,    0,    0,    0,  224,  226,  229,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          224,  224,    0,    0,  263,    0,  227,  227,  227,    0, 
            0,  227,  227,  227,  263,  227,    0,    0,    0,    0, 
            0,  226,    0,    0,  226,  227,  227,  226,    0,  226, 
            0,    0,    0,    0,  227,  227,    0,  227,  227,  227, 
          227,  227,    0,    0,    0,  226,    0,    0,  263,  227, 
            0,    0,    0,    0,    0,    0,    0,  226,    0,    0, 
            0,    0,  263,  263,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  226,    0,    0, 
            0,    0,    0,  227,    0,    0,  227,    0,    0,  227, 
            0,  227,    0,  225,    0,  222,  222,  222,    0,    0, 
          222,  222,  222,  225,  222,    0,    0,  227,    0,    0, 
            0,    0,    0,    0,  222,  222,    0,    0,    0,  227, 
            0,    0,    0,  222,  222,    0,  222,  222,  222,  222, 
          222,    0,    0,    0,    0,    0,    0,  225,  222,  227, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  225,  225,    0,    0,  265,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  265,    0,    0,    0,    0, 
            0,    0,  222,    0,    0,  222,    0,    0,  222,    0, 
          222,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  222,    0,    0,  265, 
            0,    0,    0,    0,  224,  224,  224,    0,  222,  224, 
          224,  224,    0,  224,  265,    0,    0,    0,    0,    0, 
            0,    0,    0,  224,  224,    0,    0,    0,  222,    0, 
            0,    0,  224,  224,    0,  224,  224,  224,  224,  224, 
            0,    0,    0,    0,  223,    0,    0,  224,    0,    0, 
            0,    0,    0,    0,  223,    0,    0,    0,    0,    0, 
          296,    0,    0,    0,    0,    0,  263,  263,  263,    0, 
          296,  263,  263,  263,    0,  263,    0,    0,    0,    0, 
            0,  224,    0,    0,  224,  263,  263,  224,  223,  224, 
            0,    0,    0,    0,  263,  263,    0,  263,  263,  263, 
          263,  263,  223,  223,  289,  224,    0,    0,    0,  263, 
            0,    0,    0,    0,    0,    0,    0,  224,    0,  296, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  224,    0,    0, 
            0,    0,    0,  263,    0,    0,  263,    0,    0,  263, 
            0,  263,    0,  221,    0,  225,  225,  225,    0,    0, 
          225,  225,  225,  221,  225,    0,    0,  263,    0,    0, 
            0,    0,    0,    0,  225,  225,    0,    0,    0,  263, 
            0,    0,    0,  225,  225,    0,  225,  225,  225,  225, 
          225,    0,    0,    0,    0,    0,    0,  221,  225,  263, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          294,  221,  221,    0,    0,    0,    0,  265,  265,  265, 
          294,    0,  265,  265,  265,    0,  265,    0,    0,    0, 
            0,    0,  225,    0,    0,  225,  265,  265,  225,    0, 
          225,    0,    0,    0,    0,  265,  265,    0,  265,  265, 
          265,  265,  265,    0,  290,    0,  225,    0,    0,    0, 
          265,    0,    0,    0,    0,    0,    0,    0,  225,  294, 
            0,    0,    0,  295,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  295,    0,    0,    0,    0,  225,    0, 
            0,    0,    0,   25,  265,    0,    0,  265,    0,    0, 
          265,    0,  265,   25,    0,    0,  223,  223,  223,    0, 
            0,  223,  223,  223,    0,  223,    0,  291,  265,    0, 
            0,    0,  296,  296,  296,  223,  223,  296,  296,  296, 
          265,  296,  295,  221,  223,  223,    0,  223,  223,  223, 
          223,    0,    0,  221,    0,    0,    0,    0,    0,  223, 
          265,    0,   25,  296,  296,  296,  296,  296,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  293, 
            0,    0,    0,    0,    0,    0,    0,  265,    0,  293, 
            0,    0,    0,  223,    0,    0,  223,    0,    0,  223, 
            0,  223,  221,    0,    0,    0,    0,    0,    0,  296, 
            0,    0,  296,    0,    0,    0,    0,  223,    0,    0, 
            0,    0,    0,  288,    0,  221,  221,  221,    0,  223, 
          221,  221,  221,  296,  221,    0,    0,    0,  293,    0, 
            0,    0,    0,    0,  221,  221,    0,    0,    0,  223, 
            0,    0,  307,  221,  221,    0,  221,  221,  221,  221, 
            0,    0,    0,    0,    0,  296,    0,    0,  221,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  294,  294,  294,    0,    0,  294,  294,  294, 
            0,  294,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  221,    0,    0,  221,    0,    0,  221,    0, 
          221,    0,    0,  294,  294,  294,  294,  294,    0,    0, 
            0,    0,    0,    0,    0,    0,  221,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  221,    0, 
            0,    0,    0,    0,    0,  295,  295,  295,    0,    0, 
          295,  295,  295,    0,  295,    0,    0,    0,  221,  294, 
            0,    0,  294,    0,    0,   25,   25,   25,    0,    0, 
            0,   25,   25,    0,   25,    0,  295,  295,  295,  295, 
          295,    0,    0,  294,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   25,   25,   25,   25, 
           25,    0,  129,    0,    0,  221,  221,  221,    0,    0, 
            0,  221,  221,    0,  221,  294,    0,    0,    0,    0, 
            0,    0,  295,    0,    0,  295,    0,    0,    0,    0, 
            0,    0,    0,  221,  221,    0,  221,  221,  221,  221, 
            0,  293,  293,  293,    0,   25,  295,  293,  293,    0, 
          293,  129,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   25,    0,    0,    0, 
            0,    0,  293,  293,  293,  293,  293,    0,  295,    0, 
            0,    0,    0,  514,    0,  221,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   25,    4, 
            5,    6,    0,    8,    0,    0,  221,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,   15,   16,   17, 
           18,  293,    0,    0,    0,   19,   20,   21,  238,  239, 
          240,  241,    0,    0,  117,    0,    0,    0,  221,    0, 
            0,   28,  293,    0,  242,  243,  244,  130,  245,   35, 
          246,  247,  248,  249,  284,   40,   41,   42,   43,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  293,    0,    0,    0,    0,  250, 
            0,    0,  130,    0,    0,   46,   47,    0,   48,    0, 
          285,    0,  286,    0,   50,    0,  130,    0,    0,    0, 
            0,    0,  287,    0,    0,    0,    0,   52,  288,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,    0,    0,  517,    0, 
            0,    0,    0,    0,   65,   66,  289,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  129,  129, 
          129,  129,  129,  129,  129,  129,  129,  129,  129,    0, 
            0,  129,  129,    0,  129,  129,  129,  129,  129,  129, 
          129,    0,  615,    0,    0,  129,  129,  129,  129,  129, 
          129,  129,    0,    0,  129,    0,    0,    0,    0,    0, 
          129,  129,    0,  129,  129,  129,  129,  129,  129,  129, 
          129,  129,  129,  129,    0,  129,  129,  129,  129,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          514,    0,    0,    0,    0,    0,    0,  514,  514,  129, 
            0,    0,  129,  615,    0,  129,  129,    0,  129,    0, 
          129,    0,    0,    0,  129,    0,  129,    0,    0,  129, 
            0,    0,  129,    0,  615,    0,    0,  129,  129,  129, 
          129,  129,  129,    0,    0,    0,  129,    0,  129,  129, 
            0,  129,  129,  129,  129,    0,    0,    0,  622,    0, 
            0,    0,    0,    0,  129,  129,    0,    0,    0,    0, 
            0,    0,    0,  130,  130,  130,  130,  130,  130,  130, 
          130,  130,  130,  130,    0,    0,  130,  130,    0,  130, 
          130,  130,  130,  130,  130,  130,    0,  616,    0,    0, 
          130,  130,  130,  130,  130,  130,  130,    0,    0,  130, 
            0,    0,    0,    0,    0,  130,  130,    0,  130,  130, 
          130,  130,  130,  130,  130,  130,  130,  130,  130,    0, 
          130,  130,  130,  130,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  517,    0,    0,    0,    0, 
            0,    0,  517,  517,  130,    0,    0,  130,  616,    0, 
          130,  130,    0,  130,    0,  130,    0,    0,    0,  130, 
            0,  130,    0,    0,  130,    0,    0,  130,    0,  616, 
            0,    0,  130,  130,  130,  130,  130,  130,    0,    0, 
            0,  130,    0,  130,  130,    0,  130,  130,  130,  130, 
            0,    0,    0,  316,    0,    0,    0,    0,    0,  130, 
          130,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  129,  129,  129,  129,  129,  129,  129, 
          129,  129,  129,  129,    0,    0,  129,  129,    0,  129, 
          129,  129,  129,  129,  129,  129,    0,  622,    0,    0, 
          129,  129,  129,  129,  129,  129,  129,    0,    0,  129, 
            0,    0,    0,    0,    0,  129,  129,    0,  129,  129, 
          129,  129,  130,  129,  129,  129,  129,  129,  129,    0, 
          129,  129,  129,  129,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  622,    0,    0,    0,    0, 
            0,    0,  622,  622,  129,    0,    0,  129,  618,    0, 
          129,  129,    0,  129,    0,  129,    0,    0,    0,  129, 
            0,  130,    0,    0,  129,    0,    0,  129,    0,  622, 
            0,    0,  129,  129,  129,  129,  129,  129,    0,    0, 
            0,  129,    0,  129,  129,    0,  129,  129,  129,  129, 
            0,    0,    0,  317,    0,    0,    0,    0,    0,  129, 
          129,    0,    0,    0,    0,    0,    0,    0,  130,  130, 
          130,  130,  130,  130,  130,  130,  130,  130,  130,    0, 
            0,  130,  130,    0,  130,  130,  130,  130,  130,  130, 
          130,    0,    0,    0,    0,  130,  130,  130,  130,  130, 
          130,  130,    0,    0,  130,    0,    0,    0,    0,    0, 
          130,  130,  361,  130,  130,  130,  130,    0,  130,  130, 
          130,  130,  130,  130,    0,  130,  130,  130,  130,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          316,    0,    0,    0,    0,    0,    0,  316,  316,  130, 
            0,    0,  130,  619,    0,  130,  130,    0,  130,    0, 
          130,  361,    0,    0,  130,    0,    0,    0,    0,  130, 
            0,    0,  130,    0,    0,    0,    0,  130,  130,  130, 
          130,  130,  130,    0,    0,    0,  130,    0,  130,  130, 
            0,  130,  130,  130,  130,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  130,  130,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  130,  130, 
          130,  130,  130,  130,  130,  130,  130,  130,  130,    0, 
            0,  130,  130,    0,  130,  130,  130,  130,  130,  130, 
          130,    0,    0,    0,    0,  130,  130,  130,  130,  130, 
          130,  130,  308,    0,  130,    0,    0,    0,    0,    0, 
          130,  130,    0,  130,  130,  130,  130,    0,  130,  130, 
          130,  130,  130,  130,    0,  130,  130,  130,  130,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          317,    0,    0,    0,    0,    0,    0,  317,  317,  130, 
            0,  308,  130,    0,    0,  130,  130,    0,  130,    0, 
          130,    0,    0,    0,  130,    0,    0,    0,    0,  130, 
            0,    0,  130,    0,    0,    0,    0,  130,  130,  130, 
          130,  130,  130,    0,    0,    0,  130,    0,  130,  130, 
            0,  130,  130,  130,  130,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  130,  130,    0,    0,  361,  361, 
          361,  361,  361,  361,  361,  361,  361,  361,  361,    0, 
          361,  361,  361,  361,  361,  361,  361,  361,  361,  361, 
          361,    0,    0,    0,    0,  361,  361,  361,  361,  361, 
          361,  361,  542,    0,  361,    0,    0,    0,    0,    0, 
          361,  361,    0,  361,  361,  361,  361,    0,  361,  361, 
          361,  361,  361,  361,    0,  361,  361,  361,  361,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  361, 
            0,  542,  361,    0,    0,  361,  361,    0,  361,    0, 
          361,    0,    0,    0,  361,    0,    0,    0,    0,    0, 
            0,    0,  361,    0,    0,    0,    0,  361,  361,  361, 
          361,  361,  361,    0,    0,    0,  361,    0,  361,  361, 
            0,  361,  361,  361,  361,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  361,  361,    0,    0,  308,  308, 
          308,  308,  308,  308,  308,  308,  308,  308,  308,    0, 
            0,  308,  308,    0,  308,  308,  308,  308,  308,  308, 
          308,    0,  644,    0,    0,  308,  308,  308,  308,  308, 
          308,  308,    0,    0,  308,    0,    0,    0,    0,    0, 
          308,  308,    0,  308,  308,  308,  308,    0,  308,  308, 
          308,  308,  308,  308,    0,  308,  308,  308,  308,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  644,    0,    0,    0,    0,    0,    0,    0,  308, 
            0,    0,  308,    0,    0,  308,  308,    0,  308,    0, 
          308,    0,    0,    0,  308,    0,    0,    0,    0,    0, 
            0,    0,  308,    0,   18,    0,    0,  308,  308,  308, 
          308,  308,  308,    0,    0,    0,  308,    0,  308,  308, 
            0,  308,  308,  308,  308,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  308,  308,    0,    0,  542,  542, 
          542,  542,  542,  542,  542,  542,  542,  542,  542,    0, 
            0,  542,  542,    0,  542,  542,  542,  542,  542,  542, 
          542,  644,    0,    0,    0,  542,  542,  542,  542,  542, 
          542,  542,    0,    0,  542,    0,    0,    0,    0,    0, 
          542,  542,    0,  542,  542,  542,  542,    0,  542,  542, 
          542,  542,  542,  542,    0,  542,  542,  542,  542,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          644,    0,    0,    0,    0,    0,    0,    0,    0,  542, 
            0,    0,  542,    0,    0,  542,  542,    0,  542,    0, 
          542,    0,    0,    0,  542,    0,    0,    0,    0,    0, 
            0,    0,  542,    0,    0,    0,    0,  542,  542,  542, 
          542,  542,  542,    0,    0,    0,  542,    0,  542,  542, 
            0,  542,  542,  542,  542,    0,    0,    0,  644,  644, 
          644,  644,  644,  644,  542,  542,  644,  644,  644,    0, 
            0,    0,  644,    0,  644,  644,  644,  644,  644,  644, 
          644,  259,    0,    0,    0,  644,  644,  644,  644,  644, 
          644,  644,    0,    0,  644,    0,    0,    0,    0,    0, 
          644,  644,    0,  644,  644,  644,  644,    0,  644,  644, 
          644,  644,  644,  644,    0,  644,  644,  644,  644,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          258,    0,    0,    0,    0,    0,    0,    0,    0,  644, 
            0,    0,  644,    0,    0,  644,  644,    0,  644,    0, 
          644,    0,    0,    0,  644,    0,    0,    0,    0,    0, 
            0,    0,  644,    0,    0,    0,    0,  644,  644,  644, 
          644,  644,  644,    0,    0,    0,  644,    0,  644,  644, 
            0,  644,  644,  644,  644,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  644,  644,    0,  644,  644,  644, 
          644,  644,  644,    0,    0,    0,  644,  644,    0,    0, 
          310,  644,    0,  644,  644,  644,  644,  644,  644,  644, 
      };
   }

   private static final short[] yyTable4() {
      return new short[] {

            0,    0,    0,    0,  644,  644,  644,  644,  644,  644, 
          644,    0,    0,  644,    0,    0,    0,    0,    0,  644, 
          644,    0,  644,  644,  644,  644,    0,  644,  644,  644, 
          644,  644,  644,    0,  644,  644,  644,  644,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  644,    0, 
            0,  644,    0,    0,  644,  644,    0,  644,    0,  644, 
            0,    0,    0,  644,    0,    0,    0,    0,    0,    0, 
            0,  644,    0,    0,  644,    0,  644,  644,  644,  644, 
          644,  644,    0,    0,    0,  644,    0,  644,  644,    0, 
          644,  644,  644,  644,    0,    0,    0,    0,    4,    5, 
            6,    0,    8,  644,  644,    0,    9,   10,    0,  307, 
            0,   11,    0,   12,   13,   14,   15,   16,   17,   18, 
            0,    0,    0,    0,   19,   20,   21,  238,  239,  240, 
          241,    0,    0,   26,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,    0,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,    0, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,   51,    0,    0,    0,    0,   52,   53,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,  312,  312,  312, 
            0,  312,    0,   65,   66,  312,  312,    0,    0,    0, 
          312,    0,  312,  312,  312,  312,  312,  312,  312,    0, 
            0,    0,    0,  312,  312,  312,  312,  312,  312,  312, 
            0,    0,  312,    0,    0,    0,    0,    0,    0,  312, 
            0,    0,  312,  312,  312,    0,  312,  312,  312,  312, 
          312,  312,  352,  312,  312,  312,  312,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  312,    0,    0, 
          312,    0,  310,  312,  312,    0,  312,    0,    0,    0, 
            0,    0,  312,    0,    0,    0,    0,    0,    0,    0, 
          312,    0,    0,    0,    0,  312,  312,  312,  312,  312, 
          312,    0,    0,    0,  312,    0,  312,  312,    0,  312, 
          312,  312,  312,    0,    0,    0,    4,    5,    6,    0, 
            8,    0,  312,  312,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,   15,   16,   17,   18,    0,    0, 
            0,    0,   19,   20,   21,  238,  239,  240,  241,    0, 
            0,   26,    0,    0,    0,    0,    0,    0,   28,    0, 
            0,  242,  243,  244,    0,  245,   35,  246,  247,  248, 
          249,  627,   40,   41,   42,   43,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  250,    0,    0,  130, 
            0,    0,   46,   47,    0,   48,    0,    0,    0,    0, 
            0,   50,    0,    0,    0,    0,    0,    0,    0,   51, 
            0,    0,    0,    0,   52,   53,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   65,   66,    0,    0,    4,    5,    6,    0,    8, 
            0,    0,    0,    9,   10,    0,    0,    0,   11,    0, 
           12,   13,   14,  108,  109,   17,   18,    0,    0,    0, 
            0,  110,  111,  112,  339,  340,  341,  342,    0,    0, 
          117,    0,    0,    0,    0,    0,    0,  118,    0,    0, 
          343,  344,  345,    0,  346,   35,  347,  348,  349,  350, 
            0,   40,    0,    0,  127,  472,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  351,    0,    0,  130,    0, 
            0,   46,   47,    0,   48,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   52,   53,   54,   55,   56,   57,    0, 
            0,    0,   58,    0,   59,   60,    0,   61,   62,   63, 
           64,    0,    0,    0,    4,    5,    6,    0,    8,    0, 
           65,   66,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,  108,  109,   17,   18,    0,    0,    0,    0, 
          110,  111,  112,  339,  340,  341,  342,    0,    0,  117, 
            0,    0,    0,    0,    0,    0,  118,    0,    0,  343, 
          344,  345,    0,  346,   35,  347,  348,  349,  350,    0, 
           40,    0,    0,  127,  472,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  351,    0,    0,  130,    0,    0, 
           46,   47,    0,   48,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,    0,    0,    4,    5,    6,    7,    8,    0,   65, 
           66,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,   15,   16,   17,   18,    0,    0,    0,    0,   19, 
           20,   21,   22,   23,   24,   25,    0,    0,   26,    0, 
            0,    0,    0,    0,   27,   28,   29,   30,   31,   32, 
           33,    0,   34,   35,   36,   37,   38,   39,    0,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   44,    0,    0,   45,    0,    0,   46, 
           47,    0,   48,    0,   49,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,   51,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
            0,    0,    4,    5,    6,    7,  320,    0,   65,   66, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
           15,   16,   17,   18,    0,    0,    0,    0,   19,   20, 
           21,   22,   23,   24,   25,    0,    0,   26,    0,    0, 
            0,    0,    0,   27,   28,    0,   30,   31,   32,   33, 
            0,   34,   35,   36,   37,   38,   39,    0,   40,   41, 
           42,   43,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   44,    0,    0,   45,    0,    0,   46,   47, 
            0,   48,    0,   49,    0,    0,    0,   50,    0,    0, 
            0,    0,    0,    0,    0,   51,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,    0, 
            3,    4,    5,    6,    7,    8,    0,   65,   66,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,   15, 
           16,   17,   18,    0,    0,    0,    0,   19,   20,   21, 
           22,   23,   24,   25,    0,    0,   26,    0,    0,    0, 
            0,    0,   27,   28,   29,   30,   31,   32,   33,    0, 
           34,   35,   36,   37,   38,   39,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   44,    0,    0,   45,    0,    0,   46,   47,    0, 
           48,    0,   49,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,   51,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,  319, 
            4,    5,    6,    7,  320,    0,   65,   66,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,   22, 
           23,   24,   25,    0,    0,   26,    0,    0,    0,    0, 
            0,   27,   28,    0,   30,   31,   32,   33,    0,   34, 
           35,   36,   37,   38,   39,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           44,    0,    0,  321,    0,    0,   46,   47,    0,   48, 
            0,   49,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,   51,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,  319,    4, 
            5,    6,    7,  320,    0,   65,   66,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,   15,   16,   17, 
           18,    0,    0,    0,    0,   19,   20,   21,   22,   23, 
           24,   25,    0,    0,   26,    0,    0,    0,    0,    0, 
           27,   28,    0,   30,   31,   32,   33,    0,   34,   35, 
           36,   37,   38,   39,    0,   40,   41,   42,   43,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   44, 
            0,    0,   45,    0,    0,   46,   47,    0,   48,    0, 
           49,    0,    0,    0,   50,    0,    0,    0,    0,    0, 
            0,    0,   51,    0,    0,    0,    0,   52,   53,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,    0,    0,    4,    5, 
            6,    0,    8,    0,   65,   66,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,   15,   16,   17,   18, 
            0,    0,    0,    0,   19,   20,   21,  238,  239,  240, 
          241,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,  284,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,  285, 
            0,  286,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  287,    0,    0,    0,    0,   52,  288,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    0,    4,    5, 
            6,    0,    8,   65,   66,  289,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  238,  239,  240, 
          241,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,  284,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,  712, 
            0,  286,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  287,    0,    0,    0,    0,   52,  288,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    0,  283,  283, 
          283,    0,  283,   65,   66,  289,  283,  283,    0,    0, 
            0,  283,    0,  283,  283,  283,  283,  283,  283,  283, 
            0,    0,    0,    0,  283,  283,  283,  283,  283,  283, 
          283,    0,    0,  283,    0,    0,    0,    0,    0,    0, 
          283,    0,    0,  283,  283,  283,    0,  283,  283,  283, 
          283,  283,  283,  283,  283,  283,  283,  283,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  283,    0, 
            0,  283,    0,    0,  283,  283,    0,  283,    0,  283, 
            0,  283,    0,  283,    0,    0,    0,    0,    0,    0, 
            0,  283,    0,    0,    0,    0,  283,  283,  283,  283, 
          283,  283,    0,    0,    0,  283,    0,  283,  283,    0, 
          283,  283,  283,  283,    0,    0,    0,    0,    4,    5, 
            6,    0,    8,  283,  283,  283,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  238,  239,  240, 
          241,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,  284,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,  285, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  287,    0,    0,    0,    0,   52,  288,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    0,    4,    5, 
            6,    0,    8,   65,   66,  289,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  238,  239,  240, 
          241,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,  284,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,    0, 
            0,  286,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  287,    0,    0,    0,    0,   52,  288,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    0,    4,    5, 
            6,    0,    8,   65,   66,  289,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  238,  239,  240, 
          241,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,  284,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,  712, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  287,    0,    0,    0,    0,   52,  288,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    0,    4,    5, 
            6,    0,    8,   65,   66,  289,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  238,  239,  240, 
          241,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,  284,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,    0, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  287,    0,    0,    0,    0,   52,  288,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    0,    4,    5, 
            6,    7,    8,   65,   66,  289,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,   15,   16,   17,   18, 
            0,    0,    0,    0,   19,   20,   21,   22,   23,   24, 
           25,    0,    0,   26,    0,    0,    0,    0,    0,   27, 
           28,   29,   30,   31,   32,   33,    0,   34,   35,   36, 
           37,   38,   39,    0,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   44,    0, 
            0,   45,    0,    0,   46,   47,    0,   48,    0,   49, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,   51,    0,    0,    0,    0,   52,   53,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    4,    5,    6, 
            7,    8,    0,   65,   66,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,   15,   16,   17,   18,    0, 
            0,    0,    0,   19,   20,   21,   22,   23,   24,   25, 
            0,    0,   26,    0,    0,    0,    0,    0,   27,   28, 
            0,   30,   31,   32,   33,    0,   34,   35,   36,   37, 
           38,   39,    0,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   44,    0,    0, 
           45,    0,    0,   46,   47,    0,   48,    0,   49,    0, 
            0,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
           51,    0,    0,    0,    0,   52,   53,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    4,    5,    6,    0, 
            8,    0,   65,   66,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,  108,  109,   17,   18,    0,    0, 
            0,    0,  110,  111,  112,  238,  239,  240,  241,    0, 
            0,  117,    0,    0,    0,    0,    0,    0,   28,    0, 
            0,  242,  243,  244,    0,  245,   35,  246,  247,  248, 
          249,    0,   40,   41,   42,   43,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  250,    0,    0,  130, 
          507,    0,   46,   47,    0,   48,    0,    0,    0,    0, 
            0,   50,    0,    0,    0,    0,    0,    0,    0,  287, 
            0,    0,    0,    0,   52,   53,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,    0,    0,    4,    5,    6,    0,    8, 
            0,   65,   66,    9,   10,    0,    0,    0,   11,    0, 
           12,   13,   14,   15,   16,   17,   18,    0,    0,    0, 
            0,   19,   20,   21,  238,  239,  240,  241,    0,    0, 
          117,    0,    0,    0,    0,    0,    0,   28,    0,    0, 
          242,  243,  244,    0,  245,   35,  246,  247,  248,  249, 
            0,   40,   41,   42,   43,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  250,    0,    0,  130,    0, 
            0,   46,   47,    0,   48,    0,  618,    0,    0,    0, 
           50,    0,    0,    0,    0,    0,    0,    0,  287,    0, 
            0,    0,    0,   52,   53,   54,   55,   56,   57,    0, 
            0,    0,   58,    0,   59,   60,    0,   61,   62,   63, 
           64,    0,    0,    0,    4,    5,    6,    0,    8,    0, 
           65,   66,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,  108,  109,   17,   18,    0,    0,    0,    0, 
          110,  111,  112,  238,  239,  240,  241,    0,    0,  117, 
            0,    0,    0,    0,    0,    0,   28,    0,    0,  242, 
          243,  244,    0,  245,   35,  246,  247,  248,  249,    0, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  250,    0,    0,  130,    0,    0, 
           46,   47,    0,   48,    0,  285,    0,    0,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,  287,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,    0,    0,    4,    5,    6,    0,    8,    0,   65, 
           66,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  238,  239,  240,  241,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  242,  243, 
          244,    0,  245,   35,  246,  247,  248,  249,    0,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  250,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,  618,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  287,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
            0,    0,    4,    5,    6,    0,    8,    0,   65,   66, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
          108,  109,   17,   18,    0,    0,    0,    0,  110,  111, 
          112,  238,  239,  240,  241,    0,    0,  117,    0,    0, 
            0,    0,    0,    0,   28,    0,    0,  242,  243,  244, 
            0,  245,   35,  246,  247,  248,  249,    0,   40,   41, 
           42,   43,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  250,    0,    0,  130,    0,    0,   46,   47, 
            0,   48,    0,  894,    0,    0,    0,   50,    0,    0, 
            0,    0,    0,    0,    0,  287,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,    0, 
            0,    4,    5,    6,    0,    8,    0,   65,   66,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,  108, 
          109,   17,   18,    0,    0,    0,    0,  110,  111,  112, 
          238,  239,  240,  241,    0,    0,  117,    0,    0,    0, 
            0,    0,    0,   28,    0,    0,  242,  243,  244,    0, 
          245,   35,  246,  247,  248,  249,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  250,    0,    0,  130,    0,    0,   46,   47,    0, 
           48,    0,  712,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,  287,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,    0, 
          633,  633,  633,    0,  633,    0,   65,   66,  633,  633, 
            0,    0,    0,  633,    0,  633,  633,  633,  633,  633, 
          633,  633,    0,    0,    0,    0,  633,  633,  633,  633, 
          633,  633,  633,    0,    0,  633,    0,    0,    0,    0, 
            0,    0,  633,    0,    0,  633,  633,  633,    0,  633, 
          633,  633,  633,  633,  633,    0,  633,  633,  633,  633, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          633,    0,    0,  633,  633,    0,  633,  633,    0,  633, 
            0,    0,    0,    0,    0,  633,    0,    0,    0,    0, 
            0,    0,    0,  633,    0,    0,    0,    0,  633,  633, 
          633,  633,  633,  633,    0,    0,    0,  633,    0,  633, 
          633,    0,  633,  633,  633,  633,    0,    0,    0,    4, 
            5,    6,    0,    8,    0,  633,  633,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,   15,   16,   17, 
           18,    0,    0,    0,    0,   19,   20,   21,  238,  239, 
          240,  241,    0,    0,   26,    0,    0,    0,    0,    0, 
            0,   28,    0,    0,  242,  243,  244,    0,  245,   35, 
          246,  247,  248,  249,    0,   40,   41,   42,   43,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  250, 
            0,    0,  130,    0,    0,   46,   47,    0,   48,    0, 
            0,    0,    0,    0,   50,    0,    0,    0,    0,    0, 
            0,    0,   51,    0,    0,    0,    0,   52,   53,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,    0,    0,    4,    5, 
            6,    0,    8,    0,   65,   66,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  238,  239,  240, 
          241,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,    0,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,    0, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  287,    0,    0,    0,    0,   52,   53,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    4,    5,    6, 
            0,    8,    0,   65,   66,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,   15,   16,   17,   18,    0, 
            0,    0,    0,   19,   20,   21,  238,  239,  240,  241, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  242,  243,  244,    0,  245,   35,  246,  247, 
          248,  249,    0,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,    0,    0, 
            0,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          287,    0,    0,    0,    0,   52,   53,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,  633,  633,  633,    0, 
          633,    0,   65,   66,  633,  633,    0,    0,    0,  633, 
            0,  633,  633,  633,  633,  633,  633,  633,    0,    0, 
            0,    0,  633,  633,  633,  633,  633,  633,  633,    0, 
            0,  633,    0,    0,    0,    0,    0,    0,  633,    0, 
            0,  633,  633,  633,    0,  633,  633,  633,  633,  633, 
          633,    0,  633,  633,  633,  633,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  633,    0,    0,  633, 
            0,    0,  633,  633,    0,  633,    0,    0,    0,    0, 
            0,  633,    0,    0,    0,    0,    0,    0,    0,  633, 
            0,    0,    0,    0,  633,  633,  633,  633,  633,  633, 
            0,    0,    0,  633,    0,  633,  633,    0,  633,  633, 
          633,  633,    0,    0,    0,    4,    5,    6,    0,    8, 
            0,  633,  633,    9,   10,    0,    0,    0,   11,    0, 
           12,   13,   14,  108,  109,   17,   18,    0,    0,    0, 
            0,  110,  111,  112,  113,  114,  115,  116,    0,    0, 
          117,    0,    0,    0,    0,    0,    0,  118,    0,    0, 
          119,  120,  121,    0,  122,   35,  123,  124,  125,  126, 
            0,   40,    0,    0,  127,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  128,    0,    0,    0,  129,    0,    0,  130,    0, 
            0,   46,   47,    0,   48,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   52,   53,   54,   55,   56,   57,    0, 
            0,    0,   58,    0,   59,   60,    0,   61,   62,   63, 
           64,    0,    0,    0,    4,    5,    6,    0,    8,    0, 
           65,   66,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,  108,  109,   17,   18,    0,    0,    0,    0, 
          110,  111,  112,  266,  267,  268,  269,    0,    0,  117, 
            0,    0,    0,    0,    0,    0,  118,    0,    0,  270, 
          271,  272,    0,  273,   35,  274,  275,  276,  277,    0, 
           40,    0,    0,  127,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  278,    0,    0,   45,    0,    0, 
           46,   47,    0,   48,    0,   49,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,    0,    0,    4,    5,    6,    0,    8,    0,   65, 
           66,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  339,  340,  341,  342,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,  118,    0,    0,  343,  344, 
          345,    0,  346,   35,  347,  348,  349,  350,    0,   40, 
            0,    0,  127,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  351,    0,    0,  428,    0,    0,   46, 
           47,    0,   48,    0,  429,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
            0,    0,    4,    5,    6,    0,    8,    0,   65,   66, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
          108,  109,   17,   18,    0,    0,    0,    0,  110,  111, 
          112,  113,  114,  115,  116,    0,    0,  117,    0,    0, 
            0,    0,    0,    0,  118,    0,    0,  119,  120,  121, 
            0,  122,   35,  123,  124,  125,  126,    0,   40,    0, 
            0,  127,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  129,    0,    0,  130,    0,    0,   46,   47, 
            0,   48,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,    0, 
            0,    4,    5,    6,    0,    8,    0,   65,   66,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,  108, 
          109,   17,   18,    0,    0,    0,    0,  110,  111,  112, 
          339,  340,  341,  342,    0,    0,  117,    0,    0,    0, 
            0,    0,    0,  118,    0,    0,  343,  344,  345,    0, 
          346,   35,  347,  348,  349,  350,    0,   40,    0,    0, 
          127,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  351,    0,    0,  428,    0,    0,   46,   47,    0, 
           48,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,    0, 
            4,    5,    6,    0,    8,    0,   65,   66,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  108,  109, 
           17,   18,    0,    0,    0,    0,  110,  111,  112,  113, 
          114,  115,  116,    0,    0,  117,    0,    0,    0,    0, 
            0,    0,  118,    0,    0,  119,  120,  121,    0,  960, 
           35,  123,  124,  961,  126,    0,   40,    0,    0,  127, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          962,    0,    0,  130,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,    0,    4, 
            5,    6,    0,    8,    0,   65,   66,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,  108,  109,   17, 
           18,    0,    0,    0,    0,  110,  111,  112,  989,  990, 
          991,  992,    0,    0,  117,    0,    0,    0,    0,    0, 
            0,  118,    0,    0,  993,  994,  995,    0,  996,   35, 
          997,  998,  999, 1000,    0,   40,    0,    0,  127,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0, 1001, 
            0,    0,  130,    0,    0,   46,   47,    0,   48,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   52,   53,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   65,   66,  138,  139,  140,  141, 
          142,  143,  144,  145,  146,  147,  148,  149,  150,  151, 
          152,  153,  154,  155,  156,  157,  158,  159,  160,  161, 
            0,    0,  162,  163,  164,  222,  223,  224,  225,  169, 
          170,  171,  172,  173,  174,  175,  176,  177,  178,  179, 
          180,  226,  227,  228,    0,  229,  185,  309,    0,  230, 
            0,    0,    0,  187,  188,    0,  189,  190,  191,  192, 
          193,  194,  195,    0,    0,  196,  197,    0,    0,    0, 
          198,  199,  200,  201,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  203,  204,    0, 
          205,  206,  207,  208,  209,  210,  211,  212,  213,  214, 
          215,    0,    0,  216,   52,  138,  139,  140,  141,  142, 
          143,  144,  145,  146,  147,  148,  149,  150,  151,  152, 
          153,  154,  155,  156,  157,  158,  159,  160,  161,    0, 
            0,  162,  163,  164,  222,  223,  224,  225,  169,  170, 
          171,  172,  173,  174,  175,  176,  177,  178,  179,  180, 
          226,  227,  228,    0,  229,  185,    0,    0,  230,    0, 
            0,    0,  187,  188,    0,  189,  190,  191,  192,  193, 
          194,  195,    0,    0,  196,  197,    0,    0,    0,  198, 
          199,  200,  201,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  203,  204,    0,  205, 
          206,  207,  208,  209,  210,  211,  212,  213,  214,  215, 
            0,    0,  216,   52,  138,  139,  140,  141,  142,  143, 
          144,  145,  146,  147,  148,  149,  150,  151,  152,  153, 
          154,  155,  156,  157,  158,  159,  160,  161,    0,    0, 
          162,  163,  164,  165,  166,  167,  168,  169,  170,  171, 
          172,  173,  174,  175,  176,  177,  178,  179,  180,  181, 
          182,  183,    0,  184,  185,  123,  124,  186,  126,    0, 
            0,  187,  188,    0,  189,  190,  191,  192,  193,  194, 
          195,    0,    0,  196,  197,    0,    0,    0,  198,  199, 
          200,  201,    0,    0,    0,    0,    0,    0,  202,    0, 
            0,    0,    0,    0,    0,  203,  204,    0,  205,  206, 
          207,  208,  209,  210,  211,  212,  213,  214,  215,    0, 
            0,  216,  138,  139,  140,  141,  142,  143,  144,  145, 
          146,  147,  148,  149,  150,  151,  152,  153,  154,  155, 
          156,  157,  158,  159,  160,  161,    0,    0,  162,  163, 
          164,  222,  223,  224,  225,  169,  170,  171,  172,  173, 
          174,  175,  176,  177,  178,  179,  180,  226,  227,  228, 
            0,  229,  185,  359,  360,  230,  361,    0,    0,  187, 
          188,    0,  189,  190,  191,  192,  193,  194,  195,    0, 
            0,  196,  197,    0,    0,    0,  198,  199,  200,  201, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  203,  204,    0,  205,  206,  207,  208, 
          209,  210,  211,  212,  213,  214,  215,    0,    0,  216, 
          138,  139,  140,  141,  142,  143,  144,  145,  146,  147, 
          148,  149,  150,  151,  152,  153,  154,  155,  156,  157, 
          158,  159,  160,  161,    0,    0,  162,  163,  164,  222, 
          223,  224,  225,  169,  170,  171,  172,  173,  174,  175, 
          176,  177,  178,  179,  180,  226,  227,  228,    0,  229, 
          185,    0,    0,  230,    0,    0,    0,  187,  188,    0, 
          189,  190,  191,  192,  193,  194,  195,    0,    0,  196, 
          197,    0,    0,    0,  198,  199,  200,  201,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  203,  204,    0,  205,  206,  207,  208,  209,  210, 
          211,  212,  213,  214,  215,  704,  600,  216,    0,  705, 
            0,    0,    0,  187,  188,    0,  189,  190,  191,  192, 
          193,  194,  195,    0,    0,  196,  197,    0,    0,    0, 
          198,  199,  200,  201,    0,    0,    0,    0,    0,    0, 
          303,    0,    0,    0,    0,    0,    0,  203,  204,    0, 
          205,  206,  207,  208,  209,  210,  211,  212,  213,  214, 
          215,  707,  608,  216,    0,  708,    0,    0,    0,  187, 
          188,    0,  189,  190,  191,  192,  193,  194,  195,    0, 
            0,  196,  197,    0,    0,    0,  198,  199,  200,  201, 
            0,    0,    0,    0,    0,    0,  303,    0,    0,    0, 
            0,    0,    0,  203,  204,    0,  205,  206,  207,  208, 
          209,  210,  211,  212,  213,  214,  215,  735,  600,  216, 
            0,  736,    0,    0,    0,  187,  188,    0,  189,  190, 
          191,  192,  193,  194,  195,    0,    0,  196,  197,    0, 
            0,    0,  198,  199,  200,  201,    0,    0,    0,    0, 
            0,    0,  303,    0,    0,    0,    0,    0,    0,  203, 
          204,    0,  205,  206,  207,  208,  209,  210,  211,  212, 
          213,  214,  215,  738,  608,  216,    0,  739,    0,    0, 
            0,  187,  188,    0,  189,  190,  191,  192,  193,  194, 
          195,    0,    0,  196,  197,    0,    0,    0,  198,  199, 
          200,  201,    0,    0,    0,    0,    0,    0,  303,    0, 
            0,    0,    0,    0,    0,  203,  204,    0,  205,  206, 
          207,  208,  209,  210,  211,  212,  213,  214,  215, 1062, 
          600,  216,    0, 1063,    0,    0,    0,  187,  188,    0, 
          189,  190,  191,  192,  193,  194,  195,    0,    0,  196, 
          197,    0,    0,    0,  198,  199,  200,  201,    0,    0, 
            0,    0,    0,    0,  303,    0,    0,    0,    0,    0, 
            0,  203,  204,    0,  205,  206,  207,  208,  209,  210, 
          211,  212,  213,  214,  215, 1072,  600,  216,    0, 1073, 
            0,    0,    0,  187,  188,    0,  189,  190,  191,  192, 
          193,  194,  195,    0,    0,  196,  197,    0,    0,    0, 
          198,  199,  200,  201,    0,    0,    0,    0,    0,    0, 
          303,    0,    0,    0,    0,    0,    0,  203,  204,    0, 
          205,  206,  207,  208,  209,  210,  211,  212,  213,  214, 
          215, 1075,  608,  216,    0, 1076,    0,    0,    0,  187, 
          188,    0,  189,  190,  191,  192,  193,  194,  195,    0, 
            0,  196,  197,    0,    0,    0,  198,  199,  200,  201, 
            0,    0,    0,    0,    0,    0,  303,    0,    0,    0, 
            0,    0,    0,  203,  204,    0,  205,  206,  207,  208, 
          209,  210,  211,  212,  213,  214,  215,    0,    0,  216, 
      };
   }

   private static final short[] yyCheck1() {
      return new short[] {

            2,    3,    4,    5,   26,    6,   28,    9,   10,   11, 
            4,    5,   14,   15,   16,   11,   74,   19,   49,   88, 
           14,   90,    7,    6,    7,  489,    7,  420,    0,   27, 
            0,  451,   99,   21,   21,   15,   16,  469,   10,   19, 
          646,   10,   27,   45,   27,  257,   27,   49,    3,   51, 
          129,   52,    2,    3,  447,   49,   15,   16,  427,  486, 
           19,  419,  489,  421,  528,    0,  424,  449,  474,   52, 
          463,   10,   10,   10,   10,  295,   10,   44,    9,   10, 
           11,  334,   59,  431,  477,  338,  692,   59,   47,   48, 
          448,  538,   59,   10,   59,   59,  118,   99,  509,  564, 
          706,   44,   10,  805,  462,   99,  464,   10,  104,   10, 
           10,  904,  556,   45,  752,  856,   59,  680,  106,  106, 
          478,   10,   59,   59,  112,   59,  128,  906,  130,   44, 
           10,  737,   10,  598,   15,   16,  556,  321,   19,   59, 
          956,  280,   59,  536,   10,   43,   61,  104,  136,  686, 
           10,  305,   51,  104,  512,   44,   59,   10,   59,  696, 
           10,   61,   10,  341,   44,    0,   47,   10,  345,   10, 
           59,  280,  280,  341,  282,   10,   10,  305,   10,  537, 
          627,   61,  975,   61,  253,  762,  363,    2,    3,   10, 
           11,  768,  362,   59,    9,   10,   11,  128,   44,   59, 
           15,   16,   32,   10,   19,   10,   59,   44,  328,   59, 
          279,   59,   44,   59,  658,   10,  336,  855,  325,   10, 
           10,  375,   59,  362,   59,   10,  864,   59,  308,  127, 
           45,  366,  341,   10,  310,  237,   51,   44,  658,   44, 
         1056,  338,   10, 1036,  428,   10,  315,  375,   32,  290, 
          291,   10,  359,  362,  362, 1034,  720,   32,   59,  641, 
           10,   10,  264,  265,  260,   32,   10,  657,   10,   10, 
           10,   10, 1065,  282,   59, 1016,   44,   10,  498,  306, 
          982,   32,  328,  310,  353,  352,   10,   32,  670,  335, 
          336,   59,  719,  720,   10,   61,  376,  377,  300,   61, 
          302,  303,  746,  260,  306,  868,  308,  268,  269,   10, 
          873,  313,   61,  128,   44,  130,  781,  319,  729,  321, 
          300,   61,  302,  303,  373,   10,  746,  323,  330,  378, 
          267,  267,  779,  264,  265,   59,  684,  373,  341,   10, 
          305,  300,  378,  302,    2,    3,  338,  281,  373,  373, 
          352,  290,  291,  378,  378,   44,   14,  920,  352,   44, 
           61,  350,  351,  264,  279,  342,  323,  446,  269,  319, 
          271,   10,  323,  342,   59,  306,  345,  308,  267,  279, 
          382,  383,  384,  385,  386,  387,  388,   45,  310,  321, 
           61,  363,  359,  363,  269,  359,  271,  934,  429,  330, 
          790,  791,  262,  263,  264,   44,  775,  345,  268,  269, 
          375,  271,    2,    3,    4,    5,  359,  357,  420,  300, 
           59,  302,  237,  425,   14,  427,  428,  429,  363,  431, 
          432,  837,  434,  421,  342,  429,  386,  345,  421,  359, 
          420,  424,  342,  264,  265,  447,  363,  449,  450,  264, 
          265,  382,  383,  384,  385,   45,  387,  388,  460,   49, 
          448,  463,  342, 1026,  342,  448,  468,  447, 1074,    2, 
            3,  538,  130,  305,  938,  477,  464,  473,  474,  434, 
          733,  464,  342,  463,  434,  300,  698,  302,  303,  342, 
          478,  306,  395,  308,  714,  478,  428,  477,  313,  342, 
          341,  921,  934,  363,  319,  507,  321,  509,  506,   99, 
          342,  938,   45,  359,  904,  330,  906,  899,  520,  450, 
          305,  506,  359,  506,  512,  506,  425,  486,  427,  512, 
          279,   10,  431,  432,  536,  395,  538,  305,   91,  279, 
          130,  371,  372,  375,  538,   10,  427,  342,  378,  537, 
          431,  342,  342,   10,  537,  345,  536,  342,  359,  509, 
          627,  382,  383,  384,  385,  342,  525,  382,  383,  384, 
          385,  386,  387,  388,  342,  597,  507,  342,  279,  237, 
          345,  305,   61,  342,  606,  975,  976,  371,  372,  470, 
          375,  359,  342,  342,  378,  338,   61,  130,  342,   10, 
          342,  342,  342,  342,   61,  420,  988,  375,  279,  342, 
          425,   45,  427,  428,  602,  602,  431,  432,  342,  434, 
          305,  841,  610,  610,  646,  627,  342,  615,  615,   10, 
           10,   61,  447,  627,  449,  450,  263,  264,  640,  641, 
          338,  342,  269,  639, 1034,  460, 1036,  237,  463,  271, 
           61,  375, 1045,  468,   59,  313,  644,  342,  310,  647, 
          279,  319,  477,  321,   10,  339,  305,  734,  670,  752, 
          692,  342,   44,  669,  359, 1065,   44,  752,   91,   59, 
           61,  682,  263,  264,  706,  683,  362,  317, 1046,  317, 
          375,  338,  507,  342,  509,  345,  130,    2,  683,  682, 
          683,   10,  683,  342,  237,  520,  363,  709,  310,  640, 
           10,  923,  779,   59,  772,  737,   44,  929,   44,  373, 
          803,  536,  805,  313,  799,  671,  672,  729,  386,  319, 
          805,  321,  734,  312,   44,   44,  375,  373,  317,  373, 
          734,  373,   47,   48,  306,   91,   10,  347,  341,   10, 
           59,  262,  754,  305,  756,  264,  758,  759,  760,   59, 
          719,  267,  352,  765,  766,   91,  328,  329,  264,  427, 
          428,  350,  351,  775,   44,  328,  434,  779,  709,  729, 
          313,  342,  335,  336,   44,  779,  319,  783,  321,   44, 
          342,  449,  317,  363,  362,   59,  386,   44,   59,   32, 
          279,   10,  460,  684,   15,   16,   32,  359,   19,  338, 
          468,  338,   91,   61,  279,  394,  338,  819,  338,  640, 
          822,  803,  279,  375,  349,  640,  641,  752,  353,  354, 
           44,  833,  834,  835,  308,  309,   44,  311,  428,  429, 
          264,  837,   61,  306,  434,  371,  372,  373,  306,   44, 
           59,  509,  378,  386,   44,  670,   44,  859,  860,  449, 
          759,  760,  520,  342,   44,   44,  765,  766,  279,  952, 
          460,  306,  874,  956,  949,   44,  775,  342,  468,  313, 
          874,  956,  884,  269,  264,  342,  338,  321,  709,  269, 
          338,  822,  338,   10,  709,  428,  898,  899,  279,  982, 
          338,  434,  376,  377,   91,  980,  264,  982,  363,  984, 
          752,  371,  372,  373,  729,  328,  449,   59,  378,  509, 
          922,  267,  335,  336,  926,   58,  338,  460,  922,  491, 
          520,  342,  264, 1002,  280,  468,  396,  499,  500,  754, 
           44,  756,   59,  758,  759,  760,  948,  345,  538,  264, 
          765,  766,  339,  279,  948,  517,  263,   44,  267,  338, 
          775,  342,  264,  909,  264,  807,  912,  913,  810,  915, 
          952,  917,  264, 1056,   91,  363,  509,   61,   44, 1054, 
           44, 1056,  328,  641, 1059,  884,  988,  520,   44,  335, 
          336,   44, 1067,   10,  428,  341,  978,  359,  303,  263, 
          264,  822,  328, 1005,  819, 1007,  267,  822,  264,  335, 
          336, 1013,  670,  363,  264, 1090,  362,   44,  833,  834, 
          835,   44,   44,  264,  949,   44,  460,  952,   10,  954, 
          339,  956,   44,  264,  468,   44,  306,  627,  308,  309, 
          310,  311,   59, 1045,  859,  860,  342,  264, 1050,  328, 
           91,  641, 1074,  262,  263,  264,  335,  336, 1046,  268, 
          269,  359,  271, 1046,  626, 1045,  262,  263,  264,  884, 
         1052,  729,  268,  269,   91,  271,  363,   59,  339,  363, 
          670,   10,  310,  898,  899,   44,  264,  649,  310,  300, 
           44,  302, 1038, 1039, 1040, 1041,  754,   44,  756,   44, 
          758,   44,   10,   44,   10,   44,  264,  949,  641,   91, 
          952,  926,  954,  264,  956,  420,  306,  775,  308,  309, 
          310,  311,  395,   48,   44,   91,  271, 1052,  363, 1054, 
           59, 1056,  345,  342, 1059,  341,   44,  670,   44,  729, 
         1086,  328,  447,  271,  734,   10,  342,  345,  335,  336, 
          267,  456,  742,   61,  363,   61,  342,   10,  463,   91, 
           59,  819,  724,  363,  754, 1090,  756,  363,  758,  345, 
          732,   58,  477,  988,  345,  833,  834,  835,  264,   70, 
          371,  372,  373,   91,  359,   91,  395,  378,  342,  779, 
         1005,  288, 1007,   54,   59,    5,  729,  520, 1013,  395, 
            6,  859,  860,  375,  509,  615,   59,  921,  371,  372, 
         1052,  328, 1054,  752, 1056,  378,  803, 1059,  335,  336, 
          383,  754,   10,  756,  341,  758,   91,   10, 1048,  819, 
         1045,  536,   44,  795,  796, 1050,  807,  954,   91,  952, 
          898,  899,  772,  833,  834,  835,   14, 1047, 1090,  790, 
          267,   -1,   91, 1016,   -1,   -1,   15,   16,  563,  564, 
           19,   -1,   10,  280,  922,   44,   -1,   -1,  926,  859, 
          860,   59,   -1,   -1,   44,   -1,   59,   -1,   -1,   91, 
           -1,   -1,   41,   42,  874,  267,  819,  328,   47,   48, 
           -1,   50,   51,  598,  335,  336,   44,  371,  372,  373, 
          833,  834,  835,   91,  378,  610,   -1,   -1,  898,  899, 
          615,  328,   91,   61,  379,  380,  381,  382,  335,  336, 
          754,   91,  756,   -1,  758,   -1,  859,  860,    0,   -1, 
          988,   -1,  922,  262,  263,  264,  926,   -1,   10,   -1, 
          269,  903,  647,   91,   -1,  362,  328, 1005,   -1, 1007, 
          371,  372,  373,  335,  336, 1013,   10,  378,  948,  284, 
          285,  286,  328,   -1,  289,  898,  899,   -1,   -1,  335, 
          336,  279,  280,  279,  280,  371,  372,  373,   -1,  684, 
           -1,  686,  378,  688,   -1,  819,   -1,   59,  371,  372, 
          373,  696, 1050,  926,   -1,  378,  328,   -1,  988,  833, 
          834,  835,  267,  335,  336,   59,   10,  306,   -1,  308, 
          309,  310,  311,  312,  267, 1005,   -1, 1007,  317,   -1, 
          328,   -1,  328, 1013,  729,  859,  860,  335,  336,  335, 
          336,   -1,   -1,  341,  342,  341,  342,   91,   -1,   10, 
           44,  340,  747,   -1,   -1,   -1,   -1,  752,   -1,  348, 
          349,  350,  351,   -1,  362,  988,  362,   61,   10,   -1, 
         1050,   -1,   -1,  328,  898,   -1,   -1,  279,   -1,   -1, 
          335,  336, 1005,   -1, 1007,  328,  781,   -1,   -1,  267, 
         1013,  264,  335,  336,   -1,  268,  269,   91,   59,  328, 
           -1,   -1,  926,   -1,   -1,  394,  335,  336,   -1,   -1, 
          279,  280,  427,  371,  372,  373,  431,   59,   -1,  279, 
          378,    0,  817,   -1,   -1,  820,  328, 1050,   -1,   -1, 
           91,   10,   -1,  335,  336,  284,  285,  286,  287,  341, 
          289,  279,  280,  306,   -1,  308,  309,  310,  311,   91, 
          328,  300,   -1,  302,  303,  470,   -1,  335,  336,  328, 
          855,   -1,  311,   -1,   -1,   44,  335,  336,  328,  864, 
          485,  486,  341,   -1,  489,  335,  336,  340,   -1,   58, 
           59, 1005,   61, 1007,   63,  348,   -1,   -1,   -1, 1013, 
          328,   -1,   -1,  362,   -1,   -1,   -1,  335,  336,   -1, 
          262,  263,  264,  341,  342,  267,  268,  269,   -1,  271, 
          525,   -1,   91,  528,  371,  372,  373,   -1,   -1,  281, 
           -1,  378,   -1,  267,  362,   -1, 1050,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,  934, 
          389,  390,  391,  392,  393,  394,  395,  396,  397,  398, 
          399,  400,  401,  402,  403,  404,  405,  406,  407,  408, 
          409,  410,  411,  412,  413,  414,  328,   -1,   10,   -1, 
           -1,  420,   -1,  335,  336,   -1,  425,   10,  427,   -1, 
          342,   -1,  431,  432,  328,  279,  280,    0,   -1,   -1, 
           -1,  335,  336,   -1,   -1,   -1,   10,   10,  447,   -1, 
           -1,  363,   44,  618,   -1,  752,  267,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,  463,   -1,  465,  466,  467,   61, 
           -1,  470,   -1,   -1,   -1,  267,   59,   -1,  477,   -1, 
           -1,   44,   -1,  395,  328,   -1,  485,  486,  280,   -1, 
          489,  335,  336,   -1,   -1,   59,   59,  341,  342,   91, 
         1045,   -1, 1047, 1048,   -1,   -1,   -1,   -1,   91,   59, 
          807,   -1,   -1,   -1,  513,  514,  681,  328,  362,  684, 
           -1,   -1,   -1,   -1,  335,  336,  525,   91,   44,  528, 
           -1,   10,   -1,  262,  263,  264,  328,  536,  267,  268, 
          269,   91,  271,  335,  336,  710,  317,  712,   -1,  341, 
           -1,  280,  281,  282,  719,  720,   -1,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,  306, 
          362,  308,  309,  310,  311,   91,  305,    0,  349,   -1, 
           59,   -1,  353,  354,  355,  356,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,  340,  333,  334,  335,  336,   -1,   -1, 
          339,  776,   91,  342,   -1,   -1,  345,   -1,  347,  618, 
          349,   44,  351,  788,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,  362,  363,   58,   59,   -1,   -1,   -1, 
           63,   -1,   44,   -1,   44,  306,  375,  308,  309,  310, 
          311,   -1,  949,   -1,   -1,  952,   10,  954,   -1,  956, 
           10,  293,  294,  295,  296,  297,  395,  832,  306,   -1, 
          308,  309,  310,  311,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,  681,   -1,  267,  684,   -1,  279,  280,   91, 
           44,   91,   -1,   -1,   59,   -1,   -1,   -1,   -1,  262, 
          263,  264,  340,  267,  267,  268,  269,   61,  271,   59, 
          348,  710,   -1,  712,   -1,   -1,   -1,  267,  281,  282, 
          719,  720,   -1,   -1,   -1,   -1,   91,  290,  291,  894, 
          293,  294,  295,  296,  297,   -1,  328,   91,   -1,   -1, 
           -1,   -1,   -1,  335,  336,  328,    0,   -1,   -1,  341, 
          342,   -1,  335,  336,   -1, 1052,   10, 1054,   -1, 1056, 
          759,  760, 1059,  279,  328,   -1,  765,  766,   -1,   -1, 
          362,  335,  336,  938,   -1,  774,  775,  776,  328,  342, 
           -1,  780,  345,   -1,  347,  335,  336,   -1,  267,  788, 
           44,   -1,   -1, 1090,   -1,   -1,   -1,   -1,   -1,   -1, 
          363,  280,   -1,   -1,   58,   59,   -1,   61,   -1,   63, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,   -1,   -1,   -1,   -1,   -1,  825,  826,   -1,  828, 
          829,   -1,  395,  832,   -1,   -1,   -1,   91,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,  328, 
           -1,   -1,   -1,   -1,   -1,   -1,  335,  336,  281,  282, 
           -1,   -1,  341,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,  279,  877,  279, 
           -1,   -1,  305,  362,   -1,  884,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  894,   10,   -1,   -1,   -1, 
           -1,  324,  325,   -1,   -1,   -1,  329,  330,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  280,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,  279,  328,   -1,  328,   -1, 
           44,   -1,  931,  335,  336,  335,  336,   -1,   -1,  938, 
          363,   -1,   -1,   -1,   58,   59,   -1,   61,   -1,   63, 
           -1,   -1,  375,   -1,   -1,   -1,  306,   -1,  308,  309, 
          310,  311,  312,  328,   -1,   -1,   -1,  317,   -1,   -1, 
          335,  336,  395,   -1,  328,   -1,  341,   91,  328,   -1, 
           -1,  335,  336,   -1,   -1,   -1,  336,  341,  342,   -1, 
          340,  341,   -1,   -1,  359,   -1,   -1,  362,  348,  349, 
          350,  351,   -1,  257,  258,  259,   -1,  261,  262,  263, 
          264,  265,  266,  267,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,   10,  280,  281,  282,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,  394,  299, 1045,   -1,  302,  303, 
          304,  305,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   63,  333, 
          334,  335,  336,  337,  338,  339,  340,  341,  342,  343, 
          344,  345,  346,  347,  348,  349,  350,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  362,  363, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   44,   -1, 
          374,  375,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,   -1,  365,   59,  367,   -1,  369,   10,  392,  393, 
          394,  395,  375,  257,  258,  259,   -1,  261,  262,  263, 
          264,  265,  266,  267,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,   91,  280,  281,  282,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,   -1,  299,   59,   -1,  302,  303, 
          304,  305,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   44,   91,  333, 
          334,  335,  336,  337,  338,  339,  340,  341,  342,  343, 
          344,  345,  346,  347,  348,  349,  350,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  362,  363, 
          483,  365,  366,  367,  368,  369,  370,   -1,   -1,    0, 
          374,  375,  376,  377,   91,  379,  380,  381,  382,   10, 
           -1,  306,   -1,  308,  309,  310,  311,  312,  392,  393, 
          394,  395,  317,  262,  263,  264,   -1,   -1,   -1,  268, 
          269,  317,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,  340,  341,  333,  334,   -1, 
           -1,   -1,   -1,  348,  349,  350,  351,   58,   59,  552, 
           61,   -1,   63,  349,   -1,  351,  559,  353,  354,  355, 
          356,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,  333,  334, 
           91,   -1,   -1,   -1,  280,   -1,   -1,   -1,   -1,  394, 
           -1,   -1,   -1,  342,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,  306,   -1,  308, 
          309,  310,  311,  312,  363,   -1,   -1,   -1,  317,   -1, 
           -1,   -1,   -1,   -1,  267,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   10,  335, 
          336,  340,  341,   -1,   -1,  341,  395,   -1,   -1,  348, 
          349,  350,  351,   -1,  306,   -1,  308,  309,  310,  311, 
          312,   -1,   -1,  359,   -1,  317,  362,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  279,   -1,  317,  328,   58,   59,  340,   61, 
           -1,   63,  335,  336,   -1,  394,  348,  349,  350,  351, 
          333,  334,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351,   91, 
          353,  354,  355,  356,   -1,   -1,  359,   -1,  361,   -1, 
           -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336, 
           10,   -1,  394,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,  262,  263,  264,  265,  266,  267,  268,  269,  270, 
          271,  272,  273,  274,  275,  276,  277,  278,   -1,  280, 
          281,  282,  283,  284,  285,  286,  287,  288,  289,  290, 
          291,  292,  293,  294,  295,  296,  297,   -1,  299,   59, 
           -1,  302,  303,  304,  305,  306,  307,  308,  309,  310, 
          311,  312,  313,  314,  315,  316,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           44,   -1,  333,  334,  335,  336,  337,   10,  339,  340, 
          341,  342,  343,  344,  345,  346,  347,  348,  349,  350, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,  362,  363,   -1,  365,  366,  367,  368,  369,  370, 
           44,   -1,   -1,  374,  375,  376,  377,   91,  379,  380, 
          381,  382,   -1,   -1,   -1,   -1,   59,   -1,   -1,   10, 
           -1,  392,  393,  394,  395,  257,  258,  259,   -1,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,   91,  280,  281, 
          282,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,   -1,  299,   59,   -1, 
          302,  303,  304,  305,  306,  307,  308,  309,  310,  311, 
          312,  313,  314,  315,  316,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   44, 
           91,  333,  334,  335,  336,  337,   -1,  339,  340,  341, 
          342,  343,  344,  345,  346,  347,  348,  349,  350,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          362,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,    0,  374,  375,  376,  377,   91,  379,  380,  381, 
          382,   10,  317,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          392,  393,  394,  395,   -1,   -1,   -1,   -1,  333,  334, 
           -1,   -1,   -1,   -1,   -1,   -1,  306,   -1,  308,  309, 
          310,  311,  312,   -1,  349,   44,   -1,  317,  353,  354, 
          355,  356,   -1,   -1,   -1,  279,  280,   -1,  328,   58, 
           59,   -1,   61,   -1,   63,   -1,  336,   -1,   -1,   -1, 
          340,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
          350,  351,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,  279,  280,  306,   -1,  308, 
          309,  310,  311,  312,  328,   -1,   -1,   -1,  317,   -1, 
           -1,  335,  336,   -1,   -1,   -1,   -1,  341,   -1,   -1, 
           -1,   -1,   -1,  306,  394,  308,  309,  310,  311,  312, 
           -1,  340,   -1,   -1,  317,   -1,  267,   -1,  362,   -1, 
            0,  350,  351,   -1,  328,  328,   -1,   -1,   -1,   -1, 
           10,  335,  336,  336,   -1,   -1,   -1,  340,  341,   -1, 
           -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  317,   -1,  362,   -1, 
           -1,  322,  323,   -1,   44,  394,   -1,   -1,   -1,   -1, 
           -1,   -1,  333,  334,  279,   -1,   -1,  328,   58,   59, 
           -1,   -1,   -1,   63,  335,  336,   -1,   -1,  349,   -1, 
          351,  394,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1, 
          335,  336,   10,   -1,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,  281,  282,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   59,   -1,  302,  303,  304,  305,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   44,   -1,  333,  334,  335,  336,  337,   10, 
          339,  340,  341,  342,  343,  344,  345,  346,  347,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   44,   -1,   -1,  374,  375,  376,  377,   91, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,   59,   -1, 
           -1,   10,   -1,  392,  393,  394,  395,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,  267,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   91, 
          280,  281,  282,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   -1,  299, 
           59,   -1,  302,  303,  304,  305,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   44,   91,  333,  334,  335,  336,  337,   -1,  339, 
          340,  341,  342,  343,  344,  345,  346,  347,  348,  349, 
          350,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  362,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,    0,  374,  375,  376,  377,   91,  379, 
          380,  381,  382,   10,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  392,  393,  394,  395,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  306,   -1, 
          308,  309,  310,  311,  312,   -1,   -1,   44,   -1,  317, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1, 
          328,   58,   59,   -1,   -1,   -1,   63,   -1,  336,   -1, 
           -1,   -1,  340,  341,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,  350,  351,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,  279,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1,   -1, 
           -1,   -1,   -1,  335,  336,   -1,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,  306,  394,  308,  309,  310, 
          311,  312,   -1,   -1,   -1,   -1,  317,   -1,  267,   -1, 
          362,   -1,    0,   -1,   -1,   -1,  328,  328,   -1,   -1, 
           -1,   -1,   10,  335,  336,  336,   -1,   -1,   -1,  340, 
          341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350, 
          351,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  317,  318, 
          319,  320,  321,  322,  323,  324,   44,  326,  327,   -1, 
           -1,   -1,   -1,   -1,  333,  334,  279,   -1,   -1,  328, 
           58,   59,   -1,   -1,   -1,   63,  335,  336,   -1,   -1, 
          349,   -1,  351,  394,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  328,   -1,   -1,   -1,   -1, 
           -1,   -1,  335,  336,   10,   -1,   -1,   -1,   -1,   -1, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
          267,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,  281,  282,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,   -1,  299,   59,   -1,  302,  303,  304,  305,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   44,   -1,  333,  334,  335,  336, 
          337,   10,  339,  340,  341,  342,  343,  344,  345,  346, 
          347,  348,  349,  350,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  362,  363,   -1,  365,  366, 
          367,  368,  369,  370,   44,   -1,   -1,  374,  375,  376, 
          377,   91,  379,  380,  381,  382,   -1,   -1,   -1,   -1, 
           59,   -1,   -1,   44,   -1,  392,  393,  394,  395,  257, 
          258,  259,   -1,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,   91,  280,  281,  282,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
           91,  299,   -1,   -1,  302,  303,  304,  305,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   44,   -1,  333,  334,  335,  336,  337, 
           -1,  339,  340,  341,  342,  343,  344,  345,  346,  347, 
          348,  349,  350,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  362,  363,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,    0,  374,  375,  376,  377, 
           91,  379,  380,  381,  382,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,  394,  395,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          306,   -1,  308,  309,  310,  311,  312,   -1,   -1,   44, 
           -1,  317,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279, 
          280,   -1,  328,   58,   59,   -1,   61,   -1,   63,   -1, 
          336,   -1,   -1,   -1,  340,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,  350,  351,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,  279, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1, 
           -1,   -1,   -1,   -1,   -1,  335,  336,   -1,  279,  280, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,  306,  394,  308, 
          309,  310,  311,  312,   -1,   -1,   -1,   -1,  317,   -1, 
           -1,   -1,  362,   -1,    0,   -1,   -1,   -1,  328,  328, 
           -1,   -1,   -1,   -1,   10,  335,  336,  336,   -1,   -1, 
           -1,  340,  341,   -1,   -1,   -1,   -1,  328,   -1,  348, 
          349,  350,  351,   -1,  335,  336,   -1,   -1,   -1,   -1, 
          341,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,   -1, 
           -1,  362,   -1,   59,   -1,   -1,   -1,   63,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  394,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   44,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1, 
           -1,   -1,   -1,   59,  335,  336,   10,   -1,   -1,   -1, 
           -1,   -1,  257,  258,  259,   -1,  261,  262,  263,  264, 
          265,  266,  267,  268,  269,  270,  271,  272,  273,  274, 
          275,  276,  277,  278,   -1,   91,  281,  282,  283,  284, 
          285,  286,  287,  288,  289,  290,  291,  292,  293,  294, 
          295,  296,  297,   -1,  299,   59,   -1,  302,  303,  304, 
          305,  306,  307,  308,  309,  310,  311,  312,  313,  314, 
          315,  316,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   44,   -1,  333,  334, 
          335,  336,  337,   10,  339,  340,  341,  342,  343,  344, 
          345,  346,  347,  348,  349,  350,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,   -1,  363,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
          375,  376,  377,   91,  379,  380,  381,  382,   -1,   -1, 
           -1,   -1,   59,   -1,   -1,   -1,   -1,  392,  393,  394, 
          395,  257,  258,  259,   -1,  261,  262,  263,  264,  265, 
          266,  267,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,   -1,  280,  281,  282,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,  293,  294,  295, 
          296,  297,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,  312,  313,  314,  315, 
          316,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   63,   -1,  333,  334,  335, 
          336,  337,   -1,  339,  340,   -1,  342,  343,  344,  345, 
          346,  347,  348,  349,  350,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,  362,  363,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,    0,  374,   -1, 
          376,  377,  328,  379,  380,  381,  382,   10,   -1,  335, 
          336,   -1,   -1,   -1,   -1,  341,  392,  393,  394,  395, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  306,  359,  308,  309,  310,  311,  312,   -1, 
           -1,   44,   -1,  317,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  279,   -1,   -1,  328,   -1,   59,   -1,   -1,   -1, 
           63,   -1,  336,   -1,   -1,   -1,  340,  341,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,  350,  351,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  306, 
          394,  308,  309,  310,  311,  312,   -1,   -1,   -1,   -1, 
          317,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1, 
           -1,  328,   -1,   -1,   -1,   -1,   10,   -1,   -1,  336, 
           -1,   -1,   -1,  340,  341,   -1,   -1,   -1,   -1,   -1, 
           -1,  348,  349,  350,  351,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  297, 
           -1,   -1,   -1,   -1,   -1,   59,   -1,   -1,   -1,   63, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  394,   -1,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   -1,   -1,  333,  334,   91,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  349,   -1,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,  257,  258,  259,   -1,  261,  262, 
          263,  264,  265,  266,  267,  268,  269,  270,  271,  272, 
          273,  274,  275,  276,  277,  278,   -1,  280,  281,  282, 
          283,  284,  285,  286,  287,  288,  289,  290,  291,  292, 
          293,  294,  295,  296,  297,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,  312, 
          313,  314,  315,  316,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,    0,   -1, 
          333,  334,  335,  336,  337,   -1,  339,  340,   10,  342, 
          343,  344,  345,  346,  347,  348,  349,  350,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,  362, 
          363,   -1,  365,  366,  367,  368,  369,  370,   44,   -1, 
           -1,  374,   44,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,   59,   -1,   -1,   58,   59,   -1,  392, 
          393,  394,  395,  257,  258,  259,   -1,  261,  262,  263, 
          264,  265,  266,  267,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,   91,   -1,  281,  282,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,  337,   -1,  339,  340,   -1,  342,  343, 
          344,  345,  346,  347,  348,  349,  350,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,    0,  363, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   10,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  392,  393, 
          394,  395,  306,   -1,  308,  309,  310,  311,  312,   -1, 
           -1,   -1,   44,  317,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   61, 
           -1,   63,   -1,   -1,   -1,   -1,  340,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,  350,  351,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,  280,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
          394,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,   -1,   -1,   -1,   -1,  341,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,  359,   -1,   -1,  362,   -1,   -1,   -1, 
           -1,  363,   -1,  306,  307,   -1,   59,  310,   61,   -1, 
           63,  314,  315,  375,  317,  318,  319,  320,  321,  322, 
          323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331,  332, 
          333,  334,   -1,  395,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,   -1, 
           -1,  364,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
          262,  263,  264,  265,  266,   -1,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,   -1,  280,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
          312,  313,  314,  315,  316,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   63, 
           -1,  333,  334,  335,  336,  337,  338,   10,  340,  341, 
          342,  343,  344,   -1,  346,   -1,  348,  349,  350,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          362,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   44,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   61,   -1, 
          392,  393,  394,  395,  257,  258,  259,   -1,  261,  262, 
          263,  264,  265,  266,   -1,  268,  269,  270,  271,  272, 
          273,  274,  275,  276,  277,  278,   -1,  280,   91,   -1, 
          283,  284,  285,  286,  287,  288,  289,  290,  291,  292, 
          293,  294,  295,  296,  297,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,  312, 
          313,  314,  315,  316,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   63,   -1, 
          333,  334,  335,  336,  337,  338,   -1,  340,  341,  342, 
          343,  344,   -1,  346,   -1,  348,  349,  350,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,  362, 
          363,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
            0,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  392, 
          393,  394,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  297,   -1,   -1,   -1,   -1,   -1,   59, 
           -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,   -1,  329,  330,   -1,   -1,  333, 
          334,   91,   -1,   -1,   -1,   -1,  279,  280,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,   -1, 
           -1,   -1,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,   -1,   -1,   -1,   -1,    0, 
          333,  334,   -1,   -1,   -1,  328,   -1,   -1,   -1,   10, 
           -1,   -1,  335,  336,   -1,   -1,  349,   -1,  351,  342, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362, 
           -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1, 
           61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,  333,  334, 
           91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,   -1,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278, 
      };
   }

   private static final short[] yyCheck2() {
      return new short[] {

           -1,  280,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,  337,   -1, 
           10,  340,  341,  342,  343,  344,   -1,  346,   -1,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,  865,   -1,   -1,   -1,  869,   59, 
           -1,   -1,   -1,  392,  393,  394,  395,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,   -1,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   -1, 
          280,   91,   -1,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,  337,   -1,   -1, 
          340,  341,  342,  343,  344,   -1,  346,   -1,  348,  349, 
          350,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  362,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,    0,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   10,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,  392,  393,  394,  395,   -1,   -1,   -1, 1020, 
           -1,   -1, 1023, 1024,   -1,   -1, 1027, 1028,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   44,  326,  327, 
           44,   -1,   -1,   -1,   -1,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   59,   -1,   61,   -1,   63,   61,   -1,   -1, 
           -1,  349,   -1,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,   -1,   -1,  267,   -1,   -1, 
         1081, 1082, 1083, 1084,   91,   -1,   -1,   91,   -1,   -1, 
          280,   -1,   -1,   -1, 1095,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,  328,   -1, 
           -1,   -1,   10,   -1,   -1,  335,  336,   -1,   -1,   -1, 
           -1,  341,   -1,   -1,  306,  307,   -1,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,  362,   -1,  326,  327,   44,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341, 
           -1,   59,   -1,   61,   -1,   63,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
           -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
           -1,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,   -1,  279,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,  328,   -1,  333,  334,  335,  336, 
          337,  335,  336,  340,  341,  342,  343,  344,  342,  346, 
           -1,  348,  349,  350,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  362,  363,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,    0,  379,  380,  381,  382,   -1,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,  392,  393,  394,  395,  257, 
          258,  259,   -1,  261,  262,  263,  264,  265,  266,   -1, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
           59,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,  337, 
           -1,   -1,  340,  341,  342,  343,  344,   -1,  346,   -1, 
          348,  349,  350,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,  363,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,    0,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,  394,  395,   -1,   -1, 
          306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   44, 
          326,  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   58,   59,  341,   -1,   -1,   63,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,   -1,   -1,  364,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,  281,  282,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   59, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328, 
           -1,   -1,   -1,   -1,   -1,   -1,  335,  336,  337,   -1, 
           -1,  340,   -1,  342,  343,  344,   -1,  346,   -1,  348, 
           -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,    0,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,   -1,  392,  393,  394,  395,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  280,  281,  282,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   58,   59,   -1,   61,   -1,   63, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   91,  333,  334, 
          335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,    0,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   10,  361,  362,  363,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
          375,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
          395,   -1,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   58,   59,   -1,   61,   -1,   63,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,    0,   -1,   -1,   10,   -1,   10, 
           -1,   -1,   10,  363,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,   44,   -1,  395,   -1,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   61,   -1, 
           61,   59,   -1,   59,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   91,   -1, 
           91,  305,    0,   91,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,  338,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,  349,   44,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
           58,   59,   -1,   61,   -1,   63,   -1,  262,  263,  264, 
           -1,  375,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,  395,   -1,   91,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,  338,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,   44,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,   -1,  363,   58, 
           59,   -1,   61,   -1,   63,   -1,  262,  263,  264,  267, 
          375,  267,  268,  269,   -1,  271,  279,   -1,  279,   -1, 
           -1,   -1,  280,   -1,   -1,  281,  282,   -1,   -1,   -1, 
          395,   -1,   91,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   10,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   10,  328,   -1,  328,   -1,   -1, 
          328,   -1,  335,  336,  335,  336,   -1,  335,  336,  342, 
           -1,  342,   -1,  341,   -1,   44,  342,   -1,   44,  345, 
           -1,  347,   -1,   -1,  262,  263,  264,   -1,   44,  267, 
          268,  269,   61,  271,  362,   61,   -1,  363,   -1,   59, 
           -1,   -1,   -1,  281,  282,   61,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   91,   -1,   -1,   91,   -1,  305,    0,  395, 
           -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   10,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
          338,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   58,   59,   -1,   61, 
           -1,   63,   -1,  262,  263,  264,   -1,  375,  267,  268, 
          269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,  282,   -1,   -1,   -1,  395,   -1,   91, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,  338, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   58,   59,   -1,   61,   -1, 
           63,   -1,  262,  263,  264,   -1,  375,  267,  268,  269, 
          279,  271,   -1,  279,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,  279,   -1,   -1,  395,   -1,   91,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,    0,   10,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,  328, 
           10,   -1,  328,   -1,   -1,   -1,  335,  336,   -1,  335, 
          336,   -1,  328,  342,   -1,   -1,  342,   -1,   -1,  335, 
          336,   44,  342,   44,   -1,  345,  342,   -1,   -1,   -1, 
          262,  263,  264,   -1,   44,  267,  268,  269,   61,  271, 
           61,   -1,   -1,  363,   -1,   59,   -1,   -1,   -1,  281, 
          282,   61,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   91,   -1, 
           91,   -1,   -1,  305,    0,  395,   -1,   -1,   -1,   -1, 
           -1,   91,   -1,   -1,   10,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,  338,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   44,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   58,   59,   -1,   61,   -1,   63,   -1,  262, 
          263,  264,   -1,  375,  267,  268,  269,   -1,  271,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  395,   -1,   91,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,  338,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   58,   59,   -1,   61,   -1,   63,   -1,  262,  263, 
          264,   -1,  375,  267,  268,  269,  279,  271,  279,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,  279, 
           -1,   -1,  395,   -1,   91,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,    0,   10,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,  328,   10,  328,   -1,   -1, 
           -1,   -1,  335,  336,  335,  336,   -1,   -1,  328,  342, 
           -1,  342,   -1,   -1,   -1,  335,  336,   44,  342,   44, 
           -1,  345,  342,   -1,   -1,   -1,  262,  263,  264,   -1, 
           44,  267,  268,  269,   61,  271,   61,   -1,   -1,  363, 
           -1,   59,   -1,   -1,   -1,  281,  282,   61,   -1,   -1, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   91,   -1,   91,   -1,   -1,  305, 
            0,  395,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1, 
           10,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,  338,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,   44,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   58,   59, 
           -1,   61,   -1,   63,   -1,  262,  263,  264,   -1,  375, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  395, 
           -1,   91,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,  338,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   44,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,   58,   59,   -1, 
           61,   -1,   63,   -1,  262,  263,  264,   -1,  375,  267, 
          268,  269,  279,  271,  279,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,  279,   -1,   -1,  395,   -1, 
           91,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,  328,   -1,  328,   -1,   -1,   -1,   -1,  335,  336, 
          335,  336,   -1,   -1,  328,  342,   -1,  342,   -1,   -1, 
           -1,  335,  336,   -1,  342,   -1,   -1,  345,  342,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,  363,   -1,   59,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,    0,  395,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,  338,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
           44,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1,   63, 
           -1,  262,  263,  264,   -1,  375,  267,  268,  269,   -1, 
          271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,  395,   -1,   91,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,  338,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349,   44, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   58,   59,   -1,   61,   -1,   63,   -1, 
          262,  263,  264,   -1,  375,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
           -1,   -1,   -1,   -1,  395,   -1,   91,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          342,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1, 
           -1,  363,   -1,   59,   -1,   -1,  280,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,    0,  395,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,  349,   44,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,  362,  363, 
           58,   59,   -1,   61,   -1,   63,   -1,  262,  263,  264, 
           -1,  375,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,  395,   -1,   91,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,   44,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,   -1,  363,   58, 
           59,   -1,   -1,   -1,   63,   -1,  262,  263,  264,   -1, 
          375,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  281,   -1,   -1,   -1,   -1, 
          395,   -1,   91,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  342,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,  363,   -1,   59, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0,  395, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   58,   59,   -1,   -1, 
           -1,   63,   -1,  262,  263,  264,   -1,  375,  267,  268, 
          269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,  282,   -1,   -1,   -1,  395,   -1,   91, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1, 
           63,   -1,  262,  263,  264,   -1,  375,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,   -1,   -1,   -1,   -1,  395,   -1,   91,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,  363,   -1,   -1,   59,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,    0,  395,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   44,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   58,   59,   -1,   -1,   -1,   63,   -1,  262, 
          263,  264,   -1,  375,  267,  268,  269,   -1,  271,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  395,   -1,   91,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   58,   59,   -1,   -1,   -1,   63,   -1,   -1,  262, 
          263,  264,  375,   -1,   -1,  268,  269,   -1,  271,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  395,   -1,   91,   -1,   -1,   -1,   -1,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,  342, 
           -1,   -1,   10,   -1,   -1,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
          363,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  290,  291,   44,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305, 
            0,   59,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,   44,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   -1,   59, 
           -1,   -1,   -1,   63,   -1,  262,  263,  264,   -1,  375, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  395, 
           -1,   91,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   44,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,   -1,   59,   -1, 
           -1,   -1,   63,   -1,   -1,   -1,   -1,   -1,  375,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1, 
           91,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   59,   -1,   -1,   -1,   63,   -1, 
           -1,  281,  282,   -1,  342,   -1,   -1,  345,   -1,  347, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  363,   91,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,  395,    0,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   10,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,  395,   58,   59,   -1,  290, 
          291,   63,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349,    0, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   10, 
          361,   -1,  363,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   44,  395,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   58,   59,   -1, 
           -1,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,   -1,  363,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
          395,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   44,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   58,   59,   -1, 
           -1,   -1,   63,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,  333,  334,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,    0,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   10,  361, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  375,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,   44,  395,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,   -1,   58,   59,   -1,  290, 
          291,   63,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,   -1,  329,  330, 
           -1,   -1,  333,  334,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  375,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,   -1,   -1,  395,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,   -1,   -1,   44,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   58,   59,   -1,  305,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,   -1,  329,  330, 
           -1,   -1,  333,  334,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,  375,  267,  268,  269,   -1,  271, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   10,   -1,   -1,  395,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   58, 
           59,  333,  334,   -1,   63,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  375,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  395,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   44,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   59,   -1,   61,   -1,   63,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,   -1,  329,  330,   -1,   91,  333,  334,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   -1,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  375,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1, 
           -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   44,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,  282,   -1,   -1,   59,   -1,   61,   -1, 
           63,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,   -1, 
          329,  330,   -1,   -1,  333,  334,   -1,   -1,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
          349,   -1,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   -1,  262,  263,  264,   -1, 
           -1,   -1,  268,  269,   -1,  271,  375,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  280,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  290,  291,  395,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,   -1,   -1,   -1,   -1,  342,   -1,   -1,   -1, 
           -1,   -1,   -1,  349,   44,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,  362,  363,   -1,   59, 
           -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,   -1,  268,  269,   -1,  271,  395, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,   -1,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,  338,   -1,   -1,   -1,  342, 
           -1,   -1,   44,   -1,   -1,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   59,  361,   61, 
          363,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   91, 
          326,  327,  395,   -1,   -1,  331,  332,  333,  334,   -1, 
           -1,    0,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1,   10,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,   -1,   -1,  364,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   44,   -1,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,    0,   -1,  333,  334,  335,  336,   -1,  338,   -1, 
           -1,   10,  342,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,   -1,  268,  269,   -1,  271, 
           59,   -1,   61,   -1,   63,  395,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,  338,   -1,   -1,   -1, 
          342,   -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   -1,  262,  263,  264,   -1,   -1,   -1,  268, 
          269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,  395,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,  338, 
           -1,   -1,   -1,  342,   -1,   -1,   44,   -1,   -1,   -1, 
          349,   -1,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   59,  361,   61,  363,   63,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  262,  263,  264,   -1,   -1,   -1,  268, 
          269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   91,   -1,   -1,  395,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,  338, 
           -1,   -1,   -1,  342,   -1,   -1,   44,   -1,   -1,   -1, 
          349,   -1,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   59,  361,   61,  363,   63,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  306,  307,   -1,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   91,  326,  327,  395,   -1,   -1,  331, 
      };
   }

   private static final short[] yyCheck3() {
      return new short[] {

          332,  333,  334,   -1,   -1,    0,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,   10,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
           -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,   -1, 
          268,  269,   -1,  271,   59,   -1,   61,   -1,   63,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,    0,   -1,  333,  334,  335,  336,   -1, 
          338,   -1,   -1,   10,  342,   -1,   -1,   -1,   -1,   -1, 
           -1,  349,   -1,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   44,   -1,   -1, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   59,   -1,   61,   -1,   63,  395,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
          338,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1,   -1, 
           -1,  349,   -1,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   -1,  262,  263,  264, 
           -1,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,  395,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,  338,   -1,   -1,   -1,  342,   -1,   -1, 
           44,   -1,   -1,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   59,  361,   61,  363,   63, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1, 
           -1,  268,  269,   -1,  271,   -1,   -1,   91,   -1,   -1, 
          395,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,  338,   -1,   -1,   -1,  342,   -1,   -1,   44,   -1, 
           -1,   -1,  349,   -1,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   59,  361,   61,  363,   63,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  306,  307,   -1,   -1, 
          310,   -1,   -1,   -1,  314,  315,   -1,  317,  318,  319, 
          320,  321,  322,  323,   -1,   91,  326,  327,  395,   -1, 
           -1,  331,  332,  333,  334,   -1,   -1,    0,   -1,   -1, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,   10,  348,  349, 
           -1,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   44,   -1,   -1,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   61,   -1, 
           63,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,  338,   -1,    0,   -1,  342,   -1, 
           -1,   -1,   -1,   -1,   -1,  349,   10,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1, 
           44,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,  395,   -1,   -1,   58,   59,   -1,   -1,   -1,   63, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,   -1,   -1,   -1,   -1,  342,   -1,   -1,   -1, 
           -1,   -1,   -1,  349,   -1,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   -1,  262, 
          263,  264,   -1,   -1,   -1,  268,  269,   -1,  271,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,  395, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,   -1,   -1,   -1,   -1,  342, 
           -1,   -1,   -1,   -1,   -1,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   58,   59,   -1,   -1,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,  395,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,   -1,  329,  330,   -1,   -1,  333, 
          334,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,   44,  351,   -1,   -1, 
           -1,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
           58,   59,   -1,   -1,   -1,   63,   -1,   -1,   -1,   -1, 
           -1,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314, 
          315,  395,  317,  318,  319,  320,  321,  322,  323,   -1, 
           -1,  326,  327,   -1,   -1,   -1,  331,  332,  333,  334, 
           -1,   -1,    0,   -1,   -1,   -1,  341,   -1,   -1,   -1, 
           -1,   -1,   10,  348,  349,   -1,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,   -1,   -1,  364, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           58,   59,   -1,  290,  291,   63,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,   -1,  329,  330,   -1,   -1,  333,  334,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,   -1,   -1,  351,   -1,   -1,   -1,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  375,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,  395,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   58,   59,  305,   -1,   -1, 
           63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   -1,   -1,  333,  334,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,   -1,   -1,  351,   -1,   -1,    0,   -1,   -1,  357, 
          358,  359,   -1,  361,   -1,  363,   10,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,  375,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,  395,   -1,   -1, 
           44,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   58,   59,   -1,  305,   -1,   63, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   -1,   -1,  333,  334,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,   -1,   -1,  351,   -1,   -1,   -1,   -1,   -1,  357, 
          358,  359,   -1,  361,   -1,  363,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  375,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,   -1,   58,   59,   -1,  290,  291,   63, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,   -1,  329,  330,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   10,   -1,   -1,  351,   -1, 
           -1,   -1,   -1,   -1,  357,  358,  359,   -1,  361,   -1, 
          363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,  375,  267,  268,  269,   -1,  271,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,  395,   58,   59,   -1,  290,  291,   63,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,   -1,  329,  330,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,    0,  347,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,  357,  358,  359,   -1,  361,   -1,  363, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  375,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   44,  271,   -1,   -1, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           58,   59,   -1,   -1,   -1,   63,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,   -1,  329,  330,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  357,  358,  359,   -1,  361,   -1,  363, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,  375,  267,  268,  269,   -1,  271,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   10,   -1, 
           -1,  395,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,    0,   -1,   -1,   -1,   -1, 
          395,   -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   58,   59,   -1,   -1,   -1,   63,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  357, 
          358,   -1,   -1,   -1,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           58,   59,   -1,   -1,   -1,   63,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   44,  347,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  357,  358,   -1,   58,   59, 
           -1,  363,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  375,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,    0,   -1, 
           -1,   -1,   -1,  395,   -1,   -1,  281,  282,   10,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,  262,  263,  264,   -1,  363,  267, 
          268,  269,   -1,  271,   -1,    0,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
          395,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,  324,  325,   -1,   -1, 
           -1,  329,  330,   58,   59,   -1,   -1,   -1,   63,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,    0,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   10,   -1,   -1,   -1,  375,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   -1,  395,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1, 
           -1,   -1,   -1,   -1,  324,  325,   -1,   -1,   -1,  329, 
          330,   58,   59,   -1,   -1,   -1,   63,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,    0,   -1,  363,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   10,   -1,   -1,   -1,  375,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,  395,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1, 
           -1,   -1,  324,  325,   -1,   -1,   -1,  329,  330,   58, 
           59,   -1,   -1,   -1,   63,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,  363,  267,  268,  269,   -1,  271,   -1,    0,   -1, 
           -1,   -1,   -1,  375,   -1,   -1,  281,  282,   10,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,  395,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,  324, 
          325,   -1,   -1,   -1,  329,  330,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,    0,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   10,   -1,   -1,   -1, 
          375,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
          395,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,   -1,  324,  325,   -1, 
           -1,   -1,  329,  330,   58,   59,   -1,   -1,   -1,   63, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,    0,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,  281,  282,   10,   -1,   -1,   -1,  375,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,  395,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1, 
           -1,   -1,   -1,   -1,   -1,  324,  325,   -1,   -1,   -1, 
          329,  330,   58,   59,   -1,   -1,   -1,   63,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,    0,  363,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   10,   -1,   -1,  375,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,  395,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,   -1,   -1,   44,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   58,   59,  325,   -1,   -1,   -1,  329,  330,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,   -1,  262,  263, 
          264,   -1,    0,  267,  268,  269,   -1,  271,   -1,   -1, 
           -1,  363,   10,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,   -1,  375,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,   -1,  395,   -1,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           58,   59,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,  363, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   44,   -1, 
           -1,  375,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   58,   59,   -1,   -1,   -1,   -1,   -1,  305, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,    0,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   10,  271,   -1,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  375, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   44,  305,  395, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   58,   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,    0,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   10,  271,   -1,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  375,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,  305,  395,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           58,   59,   -1,   -1,    0,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   10,  271,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,  281,  282,  345,   -1,  347, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,  363,   -1,   -1,   44,  305, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
           -1,   -1,   58,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1,   -1, 
           -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,  375, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   44,   -1,  395, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   58,   59,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,    0,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   10,  271,   -1,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  375,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,  305,  395,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           58,   59,   -1,   -1,    0,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   10,  271,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,  281,  282,  345,   -1,  347, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,  363,   -1,   -1,   44,  305, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
           -1,   -1,   58,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1,   -1, 
           -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,    0,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   10,  271,   -1,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  375, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   44,  305,  395, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   58,   59,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  363,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,  262,  263,  264,   -1,  375,  267, 
          268,  269,   -1,  271,   59,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  395,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,    0,   -1,   -1,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1, 
           10,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,  281,  282,  345,   44,  347, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   58,   59,   44,  363,   -1,   -1,   -1,  305, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1,   59, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1,   -1, 
           -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,    0,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   10,  271,   -1,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  375, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   44,  305,  395, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   58,   59,   -1,   -1,   -1,   -1,  262,  263,  264, 
           10,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,  281,  282,  345,   -1, 
          347,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   44,   -1,  363,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   59, 
           -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,  395,   -1, 
           -1,   -1,   -1,    0,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   10,   -1,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   44,  363,   -1, 
           -1,   -1,  262,  263,  264,  281,  282,  267,  268,  269, 
          375,  271,   59,    0,  290,  291,   -1,  293,  294,  295, 
          296,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,  305, 
          395,   -1,   59,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   10, 
           -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   59,   -1,   -1,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,  262,  263,  264,   -1,  375, 
          267,  268,  269,  363,  271,   -1,   -1,   -1,   59,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  395, 
           -1,   -1,   10,  290,  291,   -1,  293,  294,  295,  296, 
           -1,   -1,   -1,   -1,   -1,  395,   -1,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,  395,  339, 
           -1,   -1,  342,   -1,   -1,  262,  263,  264,   -1,   -1, 
           -1,  268,  269,   -1,  271,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  293,  294,  295,  296, 
          297,   -1,   10,   -1,   -1,  262,  263,  264,   -1,   -1, 
           -1,  268,  269,   -1,  271,  395,   -1,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
           -1,  262,  263,  264,   -1,  342,  363,  268,  269,   -1, 
          271,   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,  293,  294,  295,  296,  297,   -1,  395,   -1, 
           -1,   -1,   -1,   91,   -1,  342,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,  257, 
          258,  259,   -1,  261,   -1,   -1,  363,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,  342,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,  395,   -1, 
           -1,  299,  363,   -1,  302,  303,  304,   10,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  395,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,  350,   -1,  352,   -1,   59,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,   -1,  392,  393,  394,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  256,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,   -1, 
           -1,  269,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   -1,  301,  302,  303,  304,   10,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,  337, 
           -1,   -1,  340,  341,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   59,   -1,   -1,  357, 
           -1,   -1,  360,   -1,  362,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  256,  257,  258,  259,  260,  261,  262, 
          263,  264,  265,  266,   -1,   -1,  269,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,  280,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302, 
          303,  304,   10,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  328,   -1,   -1,   -1,   -1, 
           -1,   -1,  335,  336,  337,   -1,   -1,  340,  341,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   59,   -1,   -1,  357,   -1,   -1,  360,   -1,  362, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,  392, 
          393,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  256,  257,  258,  259,  260,  261,  262, 
          263,  264,  265,  266,   -1,   -1,  269,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,  280,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302, 
          303,  304,   10,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  328,   -1,   -1,   -1,   -1, 
           -1,   -1,  335,  336,  337,   -1,   -1,  340,  341,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   59,   -1,   -1,  357,   -1,   -1,  360,   -1,  362, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,  392, 
          393,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  256,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,   -1, 
           -1,  269,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   10,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,  337, 
           -1,   -1,  340,  341,   -1,  343,  344,   -1,  346,   -1, 
          348,   59,   -1,   -1,  352,   -1,   -1,   -1,   -1,  357, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  256,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,   -1, 
           -1,  269,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   10,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   -1,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,  337, 
           -1,   59,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,  357, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,   -1,   -1,  256,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,   -1, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   10,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   -1,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   59,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,   -1,   -1,  256,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,   -1, 
           -1,  269,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   10,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   -1,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,  362,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,   -1,   -1,  256,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,   -1, 
           -1,  269,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   10,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   -1,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,  256,  257, 
          258,  259,  260,  261,  392,  393,  264,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   10,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   -1,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,   -1,  256,  257,  258, 
          259,  260,  261,   -1,   -1,   -1,  265,  266,   -1,   -1, 
           10,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
      };
   }

   private static final short[] yyCheck4() {
      return new short[] {

           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298, 
          299,   -1,  301,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  392,  393,   -1,  265,  266,   -1,   10, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,   -1,  392,  393,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   44,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,  342,  343,  344,   -1,  346,   -1,   -1,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,   -1,  392,  393,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   44,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  392,  393,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,   -1,   -1,  316,   59,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1, 
          392,  393,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,   -1,   -1,  316,   59,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,  257,  258,  259,  260,  261,   -1,  392, 
          393,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,  298,  299,  300,  301,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,   -1,  257,  258,  259,  260,  261,   -1,  392,  393, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,  298,  299,   -1,  301,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,   -1, 
          256,  257,  258,  259,  260,  261,   -1,  392,  393,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,  300,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,  256, 
          257,  258,  259,  260,  261,   -1,  392,  393,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,   -1,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  256,  257, 
          258,  259,  260,  261,   -1,  392,  393,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   -1,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,   -1,  392,  393,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  392,  393,  394,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  392,  393,  394,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  392,  393,  394,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  392,  393,  394,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1, 
           -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  392,  393,  394,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  392,  393,  394,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,  257,  258, 
          259,  260,  261,  392,  393,  394,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298, 
          299,  300,  301,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,  257,  258,  259, 
          260,  261,   -1,  392,  393,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299, 
           -1,  301,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,   -1,  392,  393,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
          341,   -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,  392,  393,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1, 
          392,  393,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1,  392, 
          393,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,   -1,  257,  258,  259,   -1,  261,   -1,  392,  393, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,   -1,  392,  393,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
          257,  258,  259,   -1,  261,   -1,  392,  393,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,  341,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,   -1,  257, 
          258,  259,   -1,  261,   -1,  392,  393,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
           -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,   -1,  392,  393,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,   -1,  392,  393,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,   -1,  392,  393,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,  392,  393,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,   -1,   -1,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  333,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1, 
          392,  393,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,   -1,   -1,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1,  392, 
          393,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
           -1,   -1,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,   -1,  257,  258,  259,   -1,  261,   -1,  392,  393, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,   -1, 
           -1,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,   -1,  392,  393,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,   -1,   -1, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
          257,  258,  259,   -1,  261,   -1,  392,  393,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,   -1,  257, 
          258,  259,   -1,  261,   -1,  392,  393,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,   -1,   -1,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,  257,  258,  259,  260, 
          261,  262,  263,  264,  265,  266,  267,  268,  269,  270, 
          271,  272,  273,  274,  275,  276,  277,  278,  279,  280, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,  290, 
          291,  292,  293,  294,  295,  296,  297,  298,  299,  300, 
          301,  302,  303,  304,   -1,  306,  307,  308,   -1,  310, 
           -1,   -1,   -1,  314,  315,   -1,  317,  318,  319,  320, 
          321,  322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1, 
          331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,   -1,   -1,  364,  365,  257,  258,  259,  260,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,  279,  280,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,  298,  299,  300,  301, 
          302,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
           -1,   -1,  364,  365,  257,  258,  259,  260,  261,  262, 
          263,  264,  265,  266,  267,  268,  269,  270,  271,  272, 
          273,  274,  275,  276,  277,  278,  279,  280,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,  290,  291,  292, 
          293,  294,  295,  296,  297,  298,  299,  300,  301,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
           -1,  314,  315,   -1,  317,  318,  319,  320,  321,  322, 
          323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331,  332, 
          333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1, 
           -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,   -1, 
           -1,  364,  257,  258,  259,  260,  261,  262,  263,  264, 
          265,  266,  267,  268,  269,  270,  271,  272,  273,  274, 
          275,  276,  277,  278,  279,  280,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,  290,  291,  292,  293,  294, 
          295,  296,  297,  298,  299,  300,  301,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,   -1,  314, 
          315,   -1,  317,  318,  319,  320,  321,  322,  323,   -1, 
           -1,  326,  327,   -1,   -1,   -1,  331,  332,  333,  334, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,   -1,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,   -1,   -1,  364, 
          257,  258,  259,  260,  261,  262,  263,  264,  265,  266, 
          267,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,  279,  280,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,  298,  299,  300,  301,  302,  303,  304,   -1,  306, 
          307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1, 
          317,  318,  319,  320,  321,  322,  323,   -1,   -1,  326, 
          327,   -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  348,  349,   -1,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  306,  307,  364,   -1,  310, 
           -1,   -1,   -1,  314,  315,   -1,  317,  318,  319,  320, 
          321,  322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1, 
          331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1, 
          341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,  306,  307,  364,   -1,  310,   -1,   -1,   -1,  314, 
          315,   -1,  317,  318,  319,  320,  321,  322,  323,   -1, 
           -1,  326,  327,   -1,   -1,   -1,  331,  332,  333,  334, 
           -1,   -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,   -1,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,  306,  307,  364, 
           -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317,  318, 
          319,  320,  321,  322,  323,   -1,   -1,  326,  327,   -1, 
           -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1,   -1, 
           -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348, 
          349,   -1,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  306,  307,  364,   -1,  310,   -1,   -1, 
           -1,  314,  315,   -1,  317,  318,  319,  320,  321,  322, 
          323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331,  332, 
          333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1, 
           -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,  306, 
          307,  364,   -1,  310,   -1,   -1,   -1,  314,  315,   -1, 
          317,  318,  319,  320,  321,  322,  323,   -1,   -1,  326, 
          327,   -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1, 
           -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1, 
           -1,  348,  349,   -1,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  306,  307,  364,   -1,  310, 
           -1,   -1,   -1,  314,  315,   -1,  317,  318,  319,  320, 
          321,  322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1, 
          331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1, 
          341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,  306,  307,  364,   -1,  310,   -1,   -1,   -1,  314, 
          315,   -1,  317,  318,  319,  320,  321,  322,  323,   -1, 
           -1,  326,  327,   -1,   -1,   -1,  331,  332,  333,  334, 
           -1,   -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,   -1,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,   -1,   -1,  364, 
      };
   }

}
